# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "24/01/2017"


import logging
import os
import shutil
import tempfile
import unittest

from silx.gui.utils.testutils import TestCaseQt
from silx.gui import qt
from orangecontrib.tomwer.widgets.reconstruction.FtseriesOW import \
    FtseriesOW
from tomwer.core.process.reconstruction.ftseries.params.fastsetupdefineglobals import FastSetupAll
from tomwer.core.process.reconstruction.ftseries.params.paganin import PaganinMode
from tomwer.core.process.reconstruction.ftseries.params.beamgeo import BeamGeoType
from tomwer.core.process.reconstruction.ftseries.params.reconsparams import _ReconsParam
from tomwer.core.process.reconstruction.ftseries.params.ft import VolSelMode, FixedSliceMode
from tomwer.gui.qtapplicationmanager import QApplicationManager
from tomwer.synctools.ftseries import QReconsParams
from tomwer.core.utils import ftseriesutils
from tomwer.core.utils.scanutils import MockEDF


logging.disable(logging.INFO)

# Makes sure a QApplication exists
_qapp = QApplicationManager()


class TestFTSerieWidget(TestCaseQt):
    """Simple unit test to test the start/stop observation button action"""

    def setUp(self):
        super(TestFTSerieWidget, self).setUp()
        self.owftseriewidget = FtseriesOW(_connect_handler=False)
        self._reconsParams = self.owftseriewidget.recons_params

        self.ftseriewidget = self.owftseriewidget._ftserie

        self.tempdir = tempfile.mkdtemp()
        MockEDF.mockScan(self.tempdir, nRadio=5, nRecons=1, nPagRecons=0, dim=10)

        self.h5_fname = os.path.join(self.tempdir, "octave.h5")
        ftseriesutils.generateDefaultH5File(self.h5_fname)
        self.savePath = tempfile.mkstemp(suffix=".h5")[1]
        self._reconsParams.copy(FastSetupAll.DEFAULT_VALUES)
        self.owftseriewidget.setAttribute(qt.Qt.WA_DeleteOnClose)

    def tearDown(self):
        os.unlink(self.savePath)
        if os.path.isdir(self.tempdir):
            shutil.rmtree(self.tempdir)
        self.owftseriewidget.close()
        del self.owftseriewidget
        super(TestFTSerieWidget, self).tearDown()

    def testAngleOffset(self):
        self.ftseriewidget.updatePath(self.tempdir)
        self.assertTrue(
            self.ftseriewidget.getReconsParamSetEditor().isParamH5Managed('FT', 'ANGLE_OFFSET')
        )
        _qapp.processEvents()
        ft = self.ftseriewidget.getReconsParamSetEditor().getStructs()
        self.assertTrue(ft['FT']['FIXEDSLICE'] == 'middle')
        self.assertTrue(float(ft['FT']['ANGLE_OFFSET_VALUE']) == 0.0)

        self.owftseriewidget.show()

        self.ftseriewidget.save(self.savePath)
        _qapp.processEvents()

        readed = FastSetupAll()
        readed.readAll(self.h5_fname, 3.8)
        self.assertTrue(readed.structures['FT']['ANGLE_OFFSET'] == 0)

        self._reconsParams.ft['ANGLE_OFFSET_VALUE'] = 1.0
        self.assertTrue(self._reconsParams.ft['ANGLE_OFFSET'] == 1)
        self.ftseriewidget.save(self.savePath)
        _qapp.processEvents()
        readed = FastSetupAll()
        readed.readAll(self.savePath, 3.8)
        self.assertTrue(readed.structures['FT']['ANGLE_OFFSET_VALUE'] == 1)
        self.assertTrue(readed.structures['FT']['ANGLE_OFFSET'] == 1)


class TestControlsFTSeriesWidget(TestCaseQt):
    """Simple test of the control button for the FTserieWidget
    """
    def setUp(self):
        super(TestControlsFTSeriesWidget, self).setUp()
        # create octave h5 with default
        self.path = tempfile.mkdtemp()
        MockEDF.mockScan(self.path, nRadio=5, nRecons=1, nPagRecons=0, dim=10)
        # one with some modifications
        self.modifySettings = os.path.join(self.path, 'modifySettings.h5')
        ft = FastSetupAll()
        assert(ft.structures['PAGANIN']['MODE'] != 2)
        ft.structures['PAGANIN']['MODE'] = 2
        ft.writeAll(self.modifySettings, 3.8)
        # simulate an acquisition (files at least)
        self.owWidget = FtseriesOW(_connect_handler=False)
        self.widget = self.owWidget._ftserie
        self.owWidget.setAttribute(qt.Qt.WA_DeleteOnClose)

    def tearDown(self):
        self.owWidget.close()
        del self.owWidget
        if os.path.isdir(self.path):
            shutil.rmtree(self.path)
        super(TestControlsFTSeriesWidget, self).tearDown()

    def testShow(self):
        """make sure the h5file is created in the scan directory"""
        self.widget.updatePath(self.path)
        _qapp.processEvents()
        self.assertTrue(self.path == self.widget._scan.path)

    def testSave(self):
        """Make sure that the save action is correct"""
        self.widget.updatePath(self.path)
        _qapp.processEvents()
        self.assertTrue(self.path == self.widget._scan.path)

        tmpH5File = tempfile.mkstemp(suffix=".h5")[1]
        self.widget.save(tmpH5File, displayInfo=False)

        ftOri = FastSetupAll()
        originalStructures = ftOri.structures

        ftSaved = FastSetupAll()
        ftSaved.readAll(filn=self.modifySettings, targetted_octave_version=3.8)
        self.assertTrue(ftSaved.structures["PAGANIN"]["MODE"].value == 2)
        ftSaved.structures["PAGANIN"]["MODE"] = originalStructures["PAGANIN"]["MODE"]
        self.assertTrue(originalStructures.to_dict() == ftSaved.structures.to_dict())

    def testLoad(self):
        """Make sure the load action is correct"""
        self.widget.updatePath(self.path)
        _qapp.processEvents()
        self.assertTrue(self.path == self.widget._scan.path)
        self.widget.show()
        self.widget.load(self.modifySettings)

        tmpH5File = tempfile.mkstemp(suffix=".h5")[1]
        self.widget.save(tmpH5File)
        self.assertTrue(
            self.fileContainEqual(self.modifySettings, tmpH5File))
        os.unlink(tmpH5File)

    def fileContainEqual(self, f1, f2):
        ft1 = FastSetupAll()
        ft1.readAll(filn=f1, targetted_octave_version=3.8)

        ft2 = FastSetupAll()
        ft2.readAll(filn=f2, targetted_octave_version=3.8)

        return ft1.structures.to_dict() == ft2.structures.to_dict()


class TestExploreDatasetForH5(TestCaseQt):
    """Make sure that in the case the octave_FT_params.h5 exists in a directory
    this will be the one picked by the editor if the 'explore scan' option
    is activated
    """
    def setUp(self):
        super(TestExploreDatasetForH5, self).setUp()
        self._reconsParams = QReconsParams()

        self.pathWithH5 = tempfile.mkdtemp()
        self.pathWithoutH5 = tempfile.mkdtemp()
        for folder in (self.pathWithH5, self.pathWithoutH5):
            MockEDF.mockScan(folder, nRadio=5, nRecons=1, nPagRecons=0, dim=10)

        self.ft = FastSetupAll()
        assert(self.ft.structures['PAGANIN']['MODE'] != 2)
        self.ft.structures['PAGANIN']['MODE'] = 2

        self.modifySettings = os.path.join(self.pathWithH5, 'octave_FT_params.h5')
        self.ft.writeAll(self.modifySettings, 3.8)

        self.owWidget = FtseriesOW(_connect_handler=False,
                                   recons_params=self._reconsParams)
        self.widget = self.owWidget._ftserie

        self._reconsParams.copy(self.ft.DEFAULT_VALUES)

    def tearDown(self):
        if os.path.isfile(self.modifySettings):
            os.remove(self.modifySettings)
        if os.path.isdir(self.pathWithH5):
            shutil.rmtree(self.pathWithH5)
        if os.path.isdir(self.pathWithoutH5):
            shutil.rmtree(self.pathWithoutH5)
        self.owWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.owWidget.close()
        self.owWidget = None
        super(TestExploreDatasetForH5, self).tearDown()

    def testExploreInFolderWithH5(self):
        """Make sure the widget if try to explore the scan dir will load
        the reconstruction parameters from it
        """
        self.widget.setH5Exploration(True)
        self.widget.updatePath(self.pathWithH5)
        self.assertTrue(self.pathWithH5 == self.widget._scan.path)
        self.assertTrue(self._reconsParams.paganin.mode.value == 2)

    def testExploreInFolderWithoutH5(self):
        """Make sure the widget if try to explore the scan dir will load
        the reconstruction parameters from it
        """
        self.widget.setH5Exploration(False)
        self.widget.updatePath(self.pathWithoutH5)
        self.assertTrue(self.pathWithoutH5 == self.widget._scan.path)
        self.assertTrue(self._reconsParams.paganin.mode.value != 2)

    def testNoExploreInFolderWithoutH5(self):
        self.widget.setH5Exploration(False)
        self.widget.updatePath(self.pathWithH5)
        self.assertTrue(self.pathWithH5 == self.widget._scan.path)
        self.assertTrue(self._reconsParams.paganin.mode.value != 2)

    def testNoExploreInFolderWithH5(self):
        self.widget.setH5Exploration(False)
        self.widget.updatePath(self.pathWithoutH5)
        self.assertTrue(self.pathWithoutH5 == self.widget._scan.path)
        self.assertTrue(self._reconsParams.paganin.mode.value != 2)


class TestLoadSaveH5File(TestCaseQt):
    """Simple test that make sure we are able to manage correctly input h5 file
    with the following behavior :
        - if some H5 parameter are missing fron the default set of parameter
        then we are loading the default parameter
        - if the h5 file as some extra H5 parameter or structure then the
        editor should also store them.
    """
    def setUp(self):
        super(TestLoadSaveH5File, self).setUp()
        self.path = tempfile.mkdtemp()
        MockEDF.mockScan(self.path, nRadio=5, nRecons=1, nPagRecons=0, dim=10)
        self.ft = FastSetupAll()
        self.file = os.path.join(self.path, 'octave_FT_params.h5')

    def tearFown(self):
        del self.ft
        del self.file
        if os.path.isdir(self.path):
            shutil.rmtree(self.path)
        super(TestLoadSaveH5File, self).tearDown()

    def loadAndSave(self):
        # save actual FastSetup status
        self.ft.writeAll(self.file, 3.8)

        # create FTSerie, load and save dataparamValue = self.getDefaultValue(param)
        owWidgetWithout = FtseriesOW(_connect_handler=False)
        widgetWithout = owWidgetWithout._ftserie
        owWidgetWithout.setAttribute(qt.Qt.WA_DeleteOnClose)

        widgetWithout.setH5Exploration(True)
        widgetWithout.updatePath(self.path)
        widgetWithout.save(self.file)
        owWidgetWithout.close()
        # then get back what FTSerie has saved
        loader = FastSetupAll()
        loader.readAll(self.file, targetted_octave_version=3.8)
        return loader.structures

    def testDefaultFileWithExtraParameters(self):
        self.ft.structures['TEST'] = {'PARAM1': 1, 'PARAM2': 'dsadsa'}
        self.ft.structures['FT']['PARAMTATA'] = 56.3

        savedData = self.loadAndSave()
        self.assertTrue('PARAMTATA' in savedData['FT'].all_params)
        self.assertTrue(savedData['FT']['PARAMTATA'] == 56.3)


class TestSync(TestCaseQt):
    """Make sure all the tab from the editor are correctly sync with
    :class:`ReconsParam` instance"""
    def setUp(self):
        super(TestSync, self).setUp()
        self._reconsParams = QReconsParams()
        self.owWidget = FtseriesOW(_connect_handler=False)
        self.owWidget.setReconsParams(self._reconsParams)
        self.widget = self.owWidget._ftserie
        assert id(self.widget.recons_params) == id(self._reconsParams)
        assert id(self.widget.recons_params.pyhst) == id(self._reconsParams.pyhst)
        assert id(self.widget.getReconsParamSetEditor().reconsparams) == id(self._reconsParams)
        assert id(self.widget.getReconsParamSetEditor().reconsparams.pyhst) == id(self._reconsParams.pyhst)

    def tearDown(self):
        self.owWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.owWidget.close()
        self.owWidget = None
        super(TestSync, self).tearDown()

    def testFTRead(self):
        """Make sure modifications from the ReconsParam are take into account
        """
        ftWidget = self.widget.getReconsParamSetEditor()._mainWidget
        assert(self._reconsParams['FT']['DO_TEST_SLICE'] == 1)
        self._reconsParams['FT']['DO_TEST_SLICE'] = 0
        self.assertFalse(ftWidget._qcbDoTestSlice.isChecked())

        assert(self._reconsParams.to_dict()['FT']['FIXEDSLICE'] == 'middle')
        assert(self._reconsParams['FT']['FIXEDSLICE'] == FixedSliceMode.middle)
        self._reconsParams['FT']['FIXEDSLICE'] = FixedSliceMode.row_n
        self.assertTrue(ftWidget._qcbSelectSlice.currentText() == 'row n')

        assert(self._reconsParams['FT']['CORRECT_SPIKES_THRESHOLD'] == 0.04)
        self._reconsParams['FT']['CORRECT_SPIKES_THRESHOLD'] = 1.23
        self.assertTrue(float(ftWidget._qleThresholdSpikesRm.text()) == 1.23)

        assert(self._reconsParams['FT']['RINGSCORRECTION'] == 0)
        self._reconsParams['FT']['RINGSCORRECTION'] = 1
        self.assertTrue(ftWidget._qcbRingsCorrection.isChecked())

        assert(self._reconsParams['FT']['VOLSELECTION_REMEMBER'] == 0)
        self._reconsParams['FT']['VOLSELECTION_REMEMBER'] = 1
        self.assertTrue(ftWidget._qcbVolSelRemenber.isChecked())

        assert(self._reconsParams.to_dict()['FT']['VOLSELECT'] == 'total')
        assert(self._reconsParams['FT']['VOLSELECT'] == VolSelMode.total)
        self._reconsParams['FT']['VOLSELECT'] = VolSelMode.graphics
        self.assertTrue(ftWidget._qcbVolumeSelection.currentText() == 'graphics')

    def testFTWrite(self):
        """Make sure user modification from the gui are updating the
        :class:`ReconsParam` instance"""
        ftWidget = self.widget.getReconsParamSetEditor()._mainWidget

        assert(self._reconsParams['FT']['DO_TEST_SLICE'] == 1)
        ftWidget._qcbDoTestSlice.setChecked(False)
        self.qapp.processEvents()
        self.assertTrue(self._reconsParams['FT']['DO_TEST_SLICE'] == 0)
        assert(self._reconsParams['FT']['FIXEDSLICE'] == FixedSliceMode.middle)
        assert(self._reconsParams.to_dict()['FT']['FIXEDSLICE'] == 'middle')

        ftWidget._qleSliceN.setText('4')
        iRowN = ftWidget._qcbSelectSlice.findText('row n')
        assert iRowN >= 0
        ftWidget._qcbSelectSlice.setCurrentIndex(iRowN)
        self.assertEqual(self._reconsParams['FT']['FIXEDSLICE'], '4')
        self.assertEqual(self._reconsParams.to_dict()['FT']['FIXEDSLICE'], '4')

        assert(self._reconsParams['FT']['VOLOUTFILE'] == 0)
        self._reconsParams['FT']['VOLOUTFILE'] = 0
        ftWidget._setSingleVolOrStack(1)
        self.assertTrue(self._reconsParams['FT']['VOLOUTFILE'] == 1)

        assert(self._reconsParams['FT']['CORRECT_SPIKES_THRESHOLD'] == 0.04)
        ftWidget._setSpikeThreshold(1.12)
        ftWidget._qleThresholdSpikesRm.editingFinished.emit()
        self.assertTrue(self._reconsParams['FT']['CORRECT_SPIKES_THRESHOLD'] == 1.12)
        ftWidget._grpThreshold.setChecked(False)
        self.assertTrue(self._reconsParams['FT']['CORRECT_SPIKES_THRESHOLD'] == 'Inf')

        assert(self._reconsParams['FT']['RINGSCORRECTION'] == 0)
        ftWidget._qcbRingsCorrection.setChecked(True)
        self.assertTrue(self._reconsParams['FT']['RINGSCORRECTION'] == 1)

        assert(self._reconsParams['FT']['VOLSELECTION_REMEMBER'] == 0)
        ftWidget._qcbVolSelRemenber.setChecked(True)
        self.assertTrue(self._reconsParams['FT']['VOLSELECTION_REMEMBER'] == 1)

        assert(self._reconsParams.to_dict()['FT']['VOLSELECT'] == 'total')
        assert(self._reconsParams['FT']['VOLSELECT'] == VolSelMode.total)
        iSel = ftWidget._qcbVolumeSelection.findText('manual')
        assert iSel >= 0
        ftWidget._qcbVolumeSelection.setCurrentIndex(iSel)
        assert(self._reconsParams.to_dict()['FT']['VOLSELECT'] == 'manual')
        assert(self._reconsParams['FT']['VOLSELECT'] == VolSelMode.manual)

    def testFTAxisRead(self):
        ftAxisWidget = self.widget.getReconsParamSetEditor()._axisWidget

        assert(self._reconsParams['FTAXIS']['DO_AXIS_CORRECTION'] == 1)
        self._reconsParams['FTAXIS']['DO_AXIS_CORRECTION'] = 0
        self.qapp.processEvents()
        self.assertFalse(ftAxisWidget._doAxisCorrection.isChecked())

        assert(self._reconsParams['FTAXIS']['USE_TOMWER_AXIS'] == 1)
        self._reconsParams['FTAXIS']['USE_TOMWER_AXIS'] = 0
        self.assertFalse(ftAxisWidget._useTomwerAxis.isChecked())

        assert(self._reconsParams['FTAXIS']['TRY_USE_OLD_TOMWER_AXIS'] == 1)
        self._reconsParams['FTAXIS']['TRY_USE_OLD_TOMWER_AXIS'] = 0
        self.assertFalse(ftAxisWidget._useOldTomwerAxis.isChecked())

    def testDisplayRead(self):
        ftAxisWidget = self.widget.getReconsParamSetEditor()._displayWidget

        assert(self._reconsParams['FT']['SHOWPROJ'] == 0)
        self._reconsParams['FT']['SHOWPROJ'] = True
        self.qapp.processEvents()
        self.assertTrue(ftAxisWidget._qcbShowProj.isChecked())

        assert(self._reconsParams['FT']['SHOWSLICE'] == 1)
        self._reconsParams['FT']['SHOWSLICE'] = 0
        self.assertFalse(ftAxisWidget._qcbShowSlice.isChecked())

        assert(self._reconsParams['FT']['ANGLE_OFFSET'] == 0)
        assert(self._reconsParams['FT']['ANGLE_OFFSET_VALUE'] == 0.0)
        self._reconsParams['FT']['ANGLE_OFFSET_VALUE'] = 2.3
        self.assertTrue(ftAxisWidget._qleAngleOffset.text() == '2.3')

    def testDisplayWrite(self):
        displayWidget = self.widget.getReconsParamSetEditor()._displayWidget
        assert self._reconsParams['FT']['SHOWPROJ'] is False
        assert(self._reconsParams.to_dict()['FT']['SHOWPROJ'] == 0)
        displayWidget._qcbShowProj.setChecked(True)
        self.assertTrue(self._reconsParams.to_dict()['FT']['SHOWPROJ'] == 1)
        self.assertTrue(self._reconsParams['FT']['SHOWPROJ'] is True)

        assert(self._reconsParams.to_dict()['FT']['SHOWSLICE'] == 1)
        assert(self._reconsParams['FT']['SHOWSLICE'] is True)
        displayWidget._qcbShowSlice.setChecked(False)
        self.assertTrue(self._reconsParams.to_dict()['FT']['SHOWSLICE'] == 0)
        self.assertTrue(self._reconsParams['FT']['SHOWSLICE'] is False)

        assert(self._reconsParams['FT']['ANGLE_OFFSET'] is False)
        assert(self._reconsParams['FT']['ANGLE_OFFSET_VALUE'] == 0.0)
        displayWidget._qleAngleOffset.setText('2.3')
        self.qapp.processEvents()
        displayWidget._qleAngleOffset.editingFinished.emit()
        assert(self._reconsParams['FT']['ANGLE_OFFSET_VALUE'] == 2.3)
        assert(self._reconsParams.to_dict()['FT']['ANGLE_OFFSET'] == 1.0)
        displayWidget._qleAngleOffset.setText('0.0')
        displayWidget._qleAngleOffset.editingFinished.emit()
        assert(self._reconsParams.to_dict()['FT']['ANGLE_OFFSET'] == 0)
        assert(self._reconsParams.to_dict()['FT']['ANGLE_OFFSET_VALUE'] == 0.0)

    def testPaganinRead(self):
        pagWidget = self.widget.getReconsParamSetEditor()._PaganinWidget

        assert(self._reconsParams.to_dict()['PAGANIN']['MODE'] == 0)
        assert(self._reconsParams['PAGANIN']['MODE'] == PaganinMode.off)
        self._reconsParams['PAGANIN']['MODE'] = PaganinMode.multi
        self.assertTrue(pagWidget._qcbpaganin.currentText() == 'multi')

        assert(self._reconsParams['PAGANIN']['DB'] == 500.)
        self._reconsParams['PAGANIN']['DB'] = 2.3
        self.assertTrue(pagWidget._qleSigmaBeta.text() == '2.3')

        assert(self._reconsParams['PAGANIN']['DB2'] == 100.)
        self._reconsParams['PAGANIN']['DB2'] = 3.6
        self.assertTrue(pagWidget._qleSigmaBeta2.text() == '3.6')

        assert(self._reconsParams['PAGANIN']['UNSHARP_COEFF'] == 3.0)
        self._reconsParams['PAGANIN']['UNSHARP_COEFF'] = 2.6
        self.assertTrue(pagWidget._unsharp_sigma_coeff.text() == '2.6')

        assert(self._reconsParams['PAGANIN']['UNSHARP_SIGMA'] == 0.8)
        self._reconsParams['PAGANIN']['UNSHARP_SIGMA'] = 1.0
        self.assertTrue(pagWidget._unsharp_sigma_mask_value.text() == '1.0')

        assert(self._reconsParams['PAGANIN']['THRESHOLD'] == 500.0)
        self._reconsParams['PAGANIN']['THRESHOLD'] = 1.0
        self.assertTrue(pagWidget._qleThreshold.text() == '1.0')

        assert(self._reconsParams['PAGANIN']['DILATE'] == 2)
        self._reconsParams['PAGANIN']['DILATE'] = 1
        self.assertTrue(pagWidget._qleDilatation.text() == '1')

        assert(self._reconsParams['PAGANIN']['MEDIANR'] == 4)
        self._reconsParams['PAGANIN']['MEDIANR'] = 1
        self.assertTrue(pagWidget._qleMedianFilterSize.text() == '1')

        assert(self._reconsParams['PAGANIN']['MKEEP_BONE'] == 0)
        self._reconsParams['PAGANIN']['MKEEP_BONE'] = 1
        self.assertTrue(pagWidget._qcbKeepBone.isChecked())

        assert(self._reconsParams['PAGANIN']['MKEEP_SOFT'] == 0)
        self._reconsParams['PAGANIN']['MKEEP_SOFT'] = 1
        self.assertTrue(pagWidget._qcbKeepSoft.isChecked())

        assert(self._reconsParams['PAGANIN']['MKEEP_ABS'] == 0)
        self._reconsParams['PAGANIN']['MKEEP_ABS'] = 1
        self.assertTrue(pagWidget._qcbKeepAbs.isChecked())

        assert(self._reconsParams['PAGANIN']['MKEEP_CORR'] == 0)
        self._reconsParams['PAGANIN']['MKEEP_CORR'] = 1
        self.assertTrue(pagWidget._qcbKeepCorr.isChecked())

        assert(self._reconsParams['PAGANIN']['MKEEP_MASK'] == 0)
        self._reconsParams['PAGANIN']['MKEEP_MASK'] = 1
        self.assertTrue(pagWidget._qcbKeepMask.isChecked())

    def testPaganinWrite(self):
        pagWidget = self.widget.getReconsParamSetEditor()._PaganinWidget

        iMulti = pagWidget._qcbpaganin.findText('multi')
        assert iMulti >= 0
        pagWidget._qcbpaganin.setCurrentIndex(iMulti)
        self.qapp.processEvents()
        self.assertTrue(self._reconsParams.to_dict()['PAGANIN']['MODE'] == 3)
        self.assertTrue(self._reconsParams['PAGANIN']['MODE'] == PaganinMode.multi)

        assert self._reconsParams['PAGANIN']['DB'] == 500.
        pagWidget._qleSigmaBeta.setText('2.3')
        pagWidget._qleSigmaBeta.editingFinished.emit()
        self.assertEqual(self._reconsParams.to_dict()['PAGANIN']['DB'], '2.3')
        self.assertEqual(self._reconsParams['PAGANIN']['DB'], 2.3)

        assert self._reconsParams['PAGANIN']['DB2'] == 100.0
        pagWidget._qleSigmaBeta2.setText('3.6')
        pagWidget._qleSigmaBeta2.editingFinished.emit()
        self.assertEqual(self._reconsParams.to_dict()['PAGANIN']['DB2'], '3.6')
        self.assertEqual(self._reconsParams['PAGANIN']['DB2'], 3.6)

        assert self._reconsParams.to_dict()['PAGANIN']['UNSHARP_COEFF'] == 3.0
        pagWidget._unsharp_sigma_coeff.setText('1.2')
        pagWidget._unsharp_sigma_coeff.editingFinished.emit()
        assert self._reconsParams.to_dict()['PAGANIN']['UNSHARP_COEFF'] == 1.2

        assert self._reconsParams.to_dict()['PAGANIN']['UNSHARP_SIGMA'] == 0.8
        pagWidget._unsharp_sigma_mask_value.setText('1.0')
        pagWidget._unsharp_sigma_mask_value.editingFinished.emit()
        assert self._reconsParams.to_dict()['PAGANIN']['UNSHARP_SIGMA'] == 1.0

        assert self._reconsParams.to_dict()['PAGANIN']['THRESHOLD'] == 500.0
        pagWidget._qleThreshold.setText('1.0')
        pagWidget._qleThreshold.editingFinished.emit()
        assert self._reconsParams.to_dict()['PAGANIN']['THRESHOLD'] == 1.0

        assert self._reconsParams['PAGANIN']['DILATE'] == 2
        pagWidget._qleDilatation.setText('1')
        pagWidget._qleDilatation.editingFinished.emit()
        assert(self._reconsParams['PAGANIN']['DILATE'] == 1)

        assert(self._reconsParams['PAGANIN']['MEDIANR'] == 4)
        pagWidget._qleMedianFilterSize.setText('1')
        pagWidget._qleMedianFilterSize.editingFinished.emit()
        assert(self._reconsParams['PAGANIN']['MEDIANR'] == 1)

        assert(self._reconsParams['PAGANIN']['MKEEP_BONE'] == 0)
        pagWidget._qcbKeepBone.setChecked(True)
        assert(self._reconsParams['PAGANIN']['MKEEP_BONE'] == 1)

        assert(self._reconsParams['PAGANIN']['MKEEP_SOFT'] == 0)
        pagWidget._qcbKeepSoft.setChecked(True)
        assert(self._reconsParams['PAGANIN']['MKEEP_SOFT'] == 1)

        assert(self._reconsParams['PAGANIN']['MKEEP_ABS'] == 0)
        pagWidget._qcbKeepAbs.setChecked(True)
        assert(self._reconsParams['PAGANIN']['MKEEP_ABS'] == 1)

        assert(self._reconsParams['PAGANIN']['MKEEP_CORR'] == 0)
        pagWidget._qcbKeepCorr.setChecked(True)
        assert(self._reconsParams['PAGANIN']['MKEEP_CORR'] == 1)

        assert(self._reconsParams['PAGANIN']['MKEEP_MASK'] == 0)
        pagWidget._qcbKeepMask.setChecked(True)
        assert(self._reconsParams['PAGANIN']['MKEEP_MASK'] == 1)

    def testPyHSTRead(self):
        pyHSTWidget = self.widget.getReconsParamSetEditor()._PyHSTWidget
        self.assertTrue(type(self._reconsParams.pyhst) == type(pyHSTWidget._recons_params))
        self.assertTrue(id(self._reconsParams.pyhst) == id(pyHSTWidget._recons_params))

        assert(self._reconsParams['PYHSTEXE']['VERBOSE'] == 0)
        self.assertFalse(pyHSTWidget._qcbverbose.isChecked())
        self.assertTrue(id(self._reconsParams.pyhst) == id(pyHSTWidget._recons_params))
        self._reconsParams['PYHSTEXE']['VERBOSE'] = True
        self.assertTrue(id(self._reconsParams.pyhst) == id(pyHSTWidget._recons_params))

        self.qapp.processEvents()
        self.assertTrue(pyHSTWidget._qcbverbose.isChecked())

        assert(self._reconsParams['PYHSTEXE']['EXE'] == 'pyhst2')
        self._reconsParams['PYHSTEXE']['EXE'] = 'pyhst3'
        self.assertTrue(pyHSTWidget._qcbPyHSTVersion.currentText() == 'pyhst3')

        assert(self._reconsParams['PYHSTEXE']['OFFV'] == 'pyhst2')
        self._reconsParams['PYHSTEXE']['OFFV'] = 'pyhst3'
        self.assertTrue(pyHSTWidget._qlOfficalVersion.text() == 'pyhst3')

        assert(self._reconsParams['PYHSTEXE']['VERBOSE_FILE'] == 'pyhst_out.txt')
        self._reconsParams['PYHSTEXE']['VERBOSE_FILE'] = 'outToto.dsad'
        self.assertTrue(pyHSTWidget._qleVerboseFile.text() == 'outToto.dsad')

    def testPyHSTWrite(self):
        pyHSTWidget = self.widget.getReconsParamSetEditor()._PyHSTWidget
        assert(self._reconsParams['PYHSTEXE']['VERBOSE'] is False)
        pyHSTWidget._qcbverbose.setChecked(True)
        self.qapp.processEvents()
        assert(self._reconsParams['PYHSTEXE']['VERBOSE'] is True)

        assert(self._reconsParams['PYHSTEXE']['EXE'] == 'pyhst2')
        pyHSTWidget._qcbPyHSTVersion.addItem('pyhst3Toto')
        pyHSTWidget._qcbPyHSTVersion.setCurrentIndex(pyHSTWidget._qcbPyHSTVersion.count() - 1)
        assert(self._reconsParams['PYHSTEXE']['EXE'] == 'pyhst3Toto')

        assert(self._reconsParams['PYHSTEXE']['VERBOSE_FILE'] == 'pyhst_out.txt')
        pyHSTWidget._qleVerboseFile.setText('outFile.dsad')
        pyHSTWidget._qleVerboseFile.editingFinished.emit()
        assert(self._reconsParams['PYHSTEXE']['VERBOSE_FILE'] == 'outFile.dsad')

    def testBeamGeoRead(self):
        beamGeoWidget = self.widget.getReconsParamSetEditor()._beamGeoWidget

        assert(self._reconsParams.to_dict()['BEAMGEO']['TYPE'] == 'p')
        assert(self._reconsParams['BEAMGEO']['TYPE'] == BeamGeoType.parallel)
        self._reconsParams['BEAMGEO']['TYPE'] = 'f'
        self.assertTrue(beamGeoWidget._qcbType.currentText() == 'fan beam')

        assert(self._reconsParams['BEAMGEO']['SX'] == 0.)
        self._reconsParams['BEAMGEO']['SX'] = 10.2
        assert(self._reconsParams['BEAMGEO']['SX'] == 10.2)
        self.assertTrue(beamGeoWidget._qleSX.text() == '10.2')

        assert(self._reconsParams['BEAMGEO']['SY'] == 0.)
        self._reconsParams['BEAMGEO']['SY'] = 20.2
        self.assertTrue(beamGeoWidget._qleSY.text() == '20.2')

        assert(self._reconsParams['BEAMGEO']['DIST'] == 55.)
        self._reconsParams['BEAMGEO']['DIST'] = 1.0
        self.assertTrue(beamGeoWidget._qleDIST.text() == '1.0')

    def testBeamGeoWrite(self):
        beamGeoWidget = self.widget.getReconsParamSetEditor()._beamGeoWidget

        assert(self._reconsParams['BEAMGEO']['TYPE'] == BeamGeoType.parallel)
        assert(self._reconsParams.to_dict()['BEAMGEO']['TYPE'] == 'p')
        iType = beamGeoWidget._qcbType.findText('fan beam')
        assert iType >= 0
        beamGeoWidget._qcbType.setCurrentIndex(iType)
        assert(self._reconsParams['BEAMGEO']['TYPE'] == BeamGeoType.fan)
        assert(self._reconsParams.to_dict()['BEAMGEO']['TYPE'] == 'f')

        assert(self._reconsParams['BEAMGEO']['SX'] == 0.)
        beamGeoWidget._qleSX.setText('10.2')
        beamGeoWidget._qleSX.editingFinished.emit()
        assert(self._reconsParams['BEAMGEO']['SX'] == 10.2)

        assert(self._reconsParams['BEAMGEO']['SY'] == 0.)
        beamGeoWidget._qleSY.setText('11.2')
        beamGeoWidget._qleSY.editingFinished.emit()
        assert(self._reconsParams['BEAMGEO']['SY'] == 11.2)

        assert(self._reconsParams['BEAMGEO']['DIST'] == 55.)
        beamGeoWidget._qleDIST.setText('0.2')
        beamGeoWidget._qleDIST.editingFinished.emit()
        assert(self._reconsParams['BEAMGEO']['DIST'] == 0.2)

    def testExpertRead(self):
        expertWidget = self.widget.getReconsParamSetEditor()._expertWidget

        assert(self._reconsParams['FT']['NUM_PART'] == 4)
        self._reconsParams['FT']['NUM_PART'] = 2
        self.assertTrue(expertWidget._qsbNumPart.value() == 2)

        assert(self._reconsParams['FT']['VERSION'] == 'fastomo3 3.2')
        self._reconsParams['FT']['VERSION'] = 'toto'
        self.assertTrue(expertWidget._qleVersion.text() == 'toto')

        assert(self._reconsParams['FT']['DATABASE'] == 0)
        self._reconsParams['FT']['DATABASE'] = 1
        self.assertTrue(expertWidget._qcbDataBase.isChecked())

        assert(self._reconsParams['FT']['NO_CHECK'] == 0)
        self.assertFalse(expertWidget._qcbNocheck.isChecked())
        self._reconsParams['FT']['NO_CHECK'] = 1
        self.assertTrue(expertWidget._qcbNocheck.isChecked())

        assert(self._reconsParams['FT']['ZEROOFFMASK'] == 1)
        self._reconsParams['FT']['ZEROOFFMASK'] = 0
        self.assertFalse(expertWidget._qcbZeroRegionMask.isChecked())

        assert(self._reconsParams['FT']['FIXHD'] == 0)
        self._reconsParams['FT']['FIXHD'] = 1
        self.assertTrue(expertWidget._qcbFixHD.isChecked())

    def testExpertWrite(self):
        expertWidget = self.widget.getReconsParamSetEditor()._expertWidget

        assert(self._reconsParams['FT']['NUM_PART'] == 4)
        expertWidget._qsbNumPart.setValue(2)
        self.qapp.processEvents()
        assert(self._reconsParams['FT']['NUM_PART'] == 2)

        assert(self._reconsParams['FT']['DATABASE'] == 0)
        expertWidget._qcbDataBase.setChecked(True)
        assert(self._reconsParams['FT']['DATABASE'] == 1)

        assert(self._reconsParams['FT']['NO_CHECK'] == 0)
        expertWidget._qcbNocheck.setChecked(True)
        assert(self._reconsParams['FT']['NO_CHECK'] == 1)

        assert(self._reconsParams['FT']['ZEROOFFMASK'] == 1)
        expertWidget._qcbZeroRegionMask.setChecked(False)
        assert(self._reconsParams['FT']['ZEROOFFMASK'] == 0)

        assert(self._reconsParams['FT']['FIXHD'] == 0)
        expertWidget._qcbFixHD.setChecked(True)
        assert(self._reconsParams['FT']['FIXHD'] == 1)

    def testOtherRead(self):
        assert isinstance(self._reconsParams['FT'], _ReconsParam)
        self._reconsParams['FT']['MYPARAM'] = 'tata'
        assert isinstance(self._reconsParams['FT'], _ReconsParam)
        otherWidget = self.widget.getReconsParamSetEditor()._otherWidget
        widgetTata = otherWidget.paramToWidget['FT']['MYPARAM']
        assert(widgetTata.text() == 'tata')
        p = self._reconsParams['FT']
        p._set_parameter_value('MYPARAM', 'toto')
        self._reconsParams['FT']._set_parameter_value('MYPARAM', 'toto')
        self.qapp.processEvents()
        otherWidget = self.widget.getReconsParamSetEditor()._otherWidget
        widgetTata = otherWidget.paramToWidget['FT']['MYPARAM']
        self.assertTrue(widgetTata.text() == 'toto')

    def testOtherWrite(self):
        self._reconsParams['FT']['MYPARAM'] = 'tata'
        otherWidget = self.widget.getReconsParamSetEditor()._otherWidget
        widgetTata = otherWidget.paramToWidget['FT']['MYPARAM']
        widgetTata.setText('toto')
        self._reconsParams['FT']['MYPARAM'] = 'toto'


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestControlsFTSeriesWidget, TestFTSerieWidget,
               TestExploreDatasetForH5, TestLoadSaveH5File, TestSync):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == '__main__':
    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2015-2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Full tomwer test suite.

"""

__authors__ = ["H.Payno"]
__license__ = "MIT"
__date__ = "19/01/2017"

import logging
import os
import shutil

from tomwer.core.scan.edfscan import EDFTomoScan
from silx.gui import qt

from tomwer.test.utils import UtilsTest

logger = logging.getLogger(__name__)


class Simulation(qt.QThread):
    """Simulation is a simple class able to simulate an acquisition by copying
    files on a targetted directory.

    :param str: targetdir the folder where the acquisition is stored
    :param str: manipulationId the id of the simulation we want to simulate
    :param str: finalState when launched, the state to reach before stopping

    :warning: the targetted directory won't be removed or cleaned during class
        destruction. This is to be managed by callers.
    """

    advancement = {
        'not started': -1,
        'starting-s0': 0,
        'starting-s1': 1,
        'acquisitionRunning': 2,
        'acquisitionDone': 3,
        'reconstructionLaunched': 4}
    sigAdvancementChanged = qt.Signal(int)

    __definedDataset = ['test01', 'test10']

    def __init__(self, targetdir, manipulationId, finalState=4):
        assert(type(manipulationId) is str)
        assert(type(targetdir) is str)
        assert(manipulationId in self.__definedDataset)
        super(Simulation, self).__init__()

        self.targetdir = targetdir
        self.outputFolder = os.path.sep.join((targetdir, manipulationId))
        self.finalState = finalState
        self.currentState = 'not started'
        self._createFinalXML = False
        self.originalFolder, self.nbSlices, self.manipulationId = \
            self.__getOriginalDataSet(manipulationId)
        self.stopFileCreationForRunningState = int(self.nbSlices/2)
        self.srcPattern = None
        self.destPattern = None

    def __getOriginalDataSet(self, dataSetID):
        """Return paths to the requested scan
        """
        assert(dataSetID in self.__definedDataset)
        dataDir = UtilsTest.getEDFDataset(dataSetID)

        assert(os.path.isdir(dataDir))
        assert(os.path.isfile(os.path.join(dataDir, dataSetID + ".info")))
        slices = EDFTomoScan(dataDir).projections
        nbSlices = len(slices)
        manipulationID = dataSetID

        return dataDir, nbSlices, manipulationID

    def advanceTo(self, state):
        """Reset the new advancement targetted

        :param str state: the new state to reach when run will be executed
        """
        assert(state in Simulation.advancement)
        assert(type(state) is str)
        self.finalState = Simulation.advancement[state]

    def setSrcDestPatterns(self, srcPattern, destPattern):
        """
        If setted, will set the .info and .xml files into a different folder
        """
        self.srcPattern = srcPattern
        self.destPattern = destPattern
        if srcPattern is not None or destPattern is not None:
            assert(os.path.isdir(srcPattern))
            assert(os.path.isdir(destPattern))
            targettedFolder = self.outputFolder.replace(self.srcPattern, self.destPattern, 1)
            if not os.path.isdir(targettedFolder):
                os.mkdir(targettedFolder)

    def __shouldExecStep(self, step):
        """Return True if the thread should exec this step to advance taking
        into consideration is current state and his final state
        """
        return self.finalState >= self.advancement[step] and \
            (Simulation.advancement[self.currentState] +1 == self.advancement[step])

    def run(self):
        """Main function, run the acquisition through all states until
        finalState is reached
        """
        if self.__shouldExecStep('starting-s0') is True:
            logger.info('starting-s0')
            self._startAcquisition()
            self.currentState = 'starting-s0'
            self.signalCurrentState()

        if self.__shouldExecStep('starting-s1') is True:
            self.copyInitialFiles()
            logger.info('starting-s1')
            self.currentState = 'starting-s1'
            self.signalCurrentState()

        if self.__shouldExecStep('acquisitionRunning') is True:
            self._copyScans((0,
                             self.stopFileCreationForRunningState))
            logger.info('acquisitionRunning')
            self.currentState = 'acquisitionRunning'
            self.signalCurrentState()

        if self.__shouldExecStep('acquisitionDone') is True:
            self._copyScans((self.stopFileCreationForRunningState,
                             self.nbSlices))
            if self._createFinalXML is True:
                inputXMLFile = os.path.join(self.originalFolder, self.manipulationId + '.xml')
                assert(os.path.isfile(inputXMLFile))
                ouputXMLFile = os.path.join(self.outputFolder, self.manipulationId + '.xml')
                shutil.copyfile(inputXMLFile, ouputXMLFile)

            logger.info('acquisitionDone')
            self.currentState = 'acquisitionDone'
            self.signalCurrentState()

    def signalCurrentState(self):
        """Signal the actual state of the simulation
        """
        self.sigAdvancementChanged.emit(self.currentState)

    def _startAcquisition(self):
        """create needed data dir
        """
        for newFolder in (self.targetdir, self.outputFolder):
            if not os.path.exists(self.outputFolder):
                os.makedirs(self.outputFolder)

    def _copyScans(self, _slicesRange):
        """copy the .edf file from the original directory to the outputFolder

        :_slicesRange tuple: the _range of slices data we want to copy
        """
        logger.info('copying files from %s to %s' % (_slicesRange[0], _slicesRange[1]))
        for iSlice in list(range(_slicesRange[0], _slicesRange[1])):
            filename = "".join((self.manipulationId, format(iSlice, '04d'), '.edf'))
            srcFile = os.path.join(self.originalFolder, filename)
            outputFile = os.path.join(self.outputFolder, filename)
            assert(os.path.isfile(srcFile))
            assert(os.path.isdir(self.outputFolder))
            shutil.copyfile(srcFile, outputFile)

    def copyInitialFiles(self):
        """copy the .info file
        """
        assert(os.path.isdir(self.originalFolder))
        logger.info('copying initial files (.info, .xml...) from %s to %s' % (self.originalFolder, self.manipulationId))
        for extension in ('.info', '.db', '.cfg'):
            filename = "".join((self.manipulationId, extension))
            srcFile = os.path.join(self.originalFolder, filename)

            targettedFolder = self.outputFolder
            if self.srcPattern is not None or self.destPattern is not None:
                targettedFolder = self.outputFolder.replace(self.srcPattern, self.destPattern, 1)
            assert(os.path.isfile(srcFile))
            assert(os.path.isdir(targettedFolder))
            assert (os.path.isdir(self.originalFolder))
            shutil.copy2(srcFile, targettedFolder)

    def createFinalXML(self, val):
        """If activated, once all the file will be copied, this will create
        an .xml file into the output directory
        """
        self._createFinalXML = val

    def createParFile(self):
        pass

    def createReconstructedFile(self):
        pass

    def createOARJob(self):
        pass

    def createDark(self):
        pass

    def createJPG(self):
        pass

    def createVolfloat(self):
        pass

    def createVolraw(self):
        pass

    def __createFileTo(self, filePath):
        assert(type(filePath) is str)
        open(filePath, 'a').close()


def main():
    import time
    _qapp = qt.QApplication.instance() or qt.QApplication([])

    inputdir = '/tmp'
    manipulationId = 'test10'
    s = Simulation(inputdir, manipulationId, finalState=Simulation.advancement['acquisitionRunning'])
    s.start()
    while(not s.isFinished()):
        _qapp.processEvents()
        time.sleep(1)


if __name__ == '__main__':
    main()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["C. Nemoz", "H. Payno"]
__license__ = "MIT"
__date__ = "07/12/2016"

import logging
from Orange.widgets import widget, gui
from Orange.widgets import settings
from Orange.widgets.widget import Output, Input
from silx.gui import qt
from orangecontrib.tomwer.orange.settings import CallbackSettingsHandler
from tomwer.core.process.datatransfert import FolderTransfert, \
    logger as FTLogger
from tomwer.web.client import OWClient
from tomwer.gui.control.datatransfert import DataTransfertSelector
from tomwer.core.scan.scanfactory import ScanFactory
from tomwer.core.scan.scanbase import TomwerScanBase

logger = logging.getLogger(__name__)


class DataTransfertOW(widget.OWWidget, FolderTransfert, OWClient):
    """
    A simple widget managing the copy of an incoming folder to an other one

    :param parent: the parent widget
    """
    name = "data transfert"
    id = "orange.widgets.tomwer.foldertransfert"
    description = "This widget insure data transfert of the received data "
    description += "to the given directory"
    icon = "icons/folder-transfert.svg"
    priority = 30
    category = "esrfWidgets"
    keywords = ["tomography", "transfert", "cp", "copy", "move", "file",
                "tomwer", 'folder']

    settingsHandler = CallbackSettingsHandler()

    want_main_area = True
    resizing_enabled = True
    compress_signal = False

    dest_dir = settings.Setting(str())
    """Parameters directly editabled from the TOFU interface"""

    scanready = qt.Signal(TomwerScanBase)

    assert len(FolderTransfert.inputs) == 1
    class Inputs:
        data_in = Input(name=FolderTransfert.inputs[0].name,
                        type=FolderTransfert.inputs[0].type,
                        doc=FolderTransfert.inputs[0].doc)

    assert len(FolderTransfert.outputs) == 1
    class Outputs:
        data_out = Output(name=FolderTransfert.outputs[0].name,
                          type=FolderTransfert.outputs[0].type,
                          doc=FolderTransfert.outputs[0].doc)

    def __init__(self, parent=None):
        FolderTransfert.__init__(self)
        widget.OWWidget.__init__(self, parent)
        OWClient.__init__(self, (logger, FTLogger))

        # define GUI
        self._widget = DataTransfertSelector(parent=self,
                                             rnice_option=True,
                                             default_root_folder=self._getDefaultOutputDir())
        self._layout = gui.vBox(self.mainArea, self.name).layout()
        self._layout.addWidget(self._widget)

        # signal / SLOT connection
        # self._widget.sigSelectionChanged.connect(self.setDestDir)
        self.settingsHandler.addCallback(self._updateSettingsVals)

        # setting configuration
        if self.dest_dir != '':
            self._widget.setFolder(self.dest_dir)

    def _requestFolder(self):
        """Launch a QFileDialog to ask the user the output directory"""
        dialog = qt.QFileDialog(self)
        dialog.setWindowTitle("Destination folder")
        dialog.setModal(1)
        dialog.setFileMode(qt.QFileDialog.DirectoryOnly)

        if not dialog.exec_():
            dialog.close()
            return None

        return dialog.selectedFiles()[0]

    def signalTransfertOk(self, input_scan, output_scan):
        if output_scan is None:
            return
        assert isinstance(output_scan, TomwerScanBase)
        self.Outputs.data_out.send(output_scan)
        self.scanready.emit(output_scan)

    def _updateSettingsVals(self):
        """function used to update the settings values"""
        self.dest_dir = self._destDir

    @Inputs.data_in
    def process(self, scan, move=False, force=True, noRsync=False):
        if scan is None:
            return
        FolderTransfert.process(self, scan=scan, move=move, force=force,
                                noRsync=noRsync)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sys
import argparse
logging.basicConfig()
_logger = logging.getLogger(__name__)


def main(argv):
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        '--all',
        dest="all",
        action="store_true",
        default=False,
        help='Transfert all scans found, even the unfinished one')
    parser.add_argument(
        '--delete-aborted',
        dest="delete_aborted",
        action="store_true",
        default=False,
        help='Remove aborted scans (both in lbsram and rnice if any found)')
    parser.add_argument(
        '--loop',
        dest="looping",
        action="store_true",
        default=False,
        help='detection loop on the root folder until stopped')
    parser.add_argument(
        '-r', '--root',
        default="/lbsram",
        help='Define the root directory where to look for scans')
    options = parser.parse_args(argv[1:])

    # print(options are...)
    # .. warning: this code should work without any qt code.
    pass


if __name__ == '__main__':
    main(sys.argv)



# /*##########################################################################
# Copyright (C) 2017 European Synchrotron Radiation Facility
#
# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
# the ESRF by the Software group.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/
"""Some widget construction to check if a sample moved"""

__author__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "01/06/2018"


from silx.gui import qt
from silx.io import dictdump

from tomwer.core.log import TomwerLogger
from tomwer.core.process.reconstruction.lamino.tofu import SCAN_TYPES, SCAN_TYPES_I, FFCWhen
from tomwer.core.process.reconstruction.lamino.tofu import _retrieve_opts_recons_cmd
from tomwer.core.utils.char import PSI_CHAR
from tomwer.gui import icons
from tomwer.gui.reconstruction.lamino.tofu import settings
from .TofuOptionLoader import _TofuOptionLoader, _getterSetter
from .projections import InputProjectionsScrollArea
from .tofuexpert import ExpertTofuWidget
from .tofuoutput import OutputTofuWidget

_logger = TomwerLogger(__name__)


class TofuWindow(qt.QMainWindow):
    """
    Widget used to interface lamino reconstruction using tofu
    """
    def __init__(self, parent):
        qt.QMainWindow.__init__(self, parent)
        self.setWindowFlags(qt.Qt.Widget)
        # main widget
        self._mainWidget = TofuWidget(parent=self)
        self.setCentralWidget(self._mainWidget)

        # expose API
        self.save = self._mainWidget.save
        self.load = self._mainWidget.load
        self.loadFromScan = self._mainWidget.loadFromScan
        self.getParameters = self._mainWidget.getParameters
        self.setParameters = self._mainWidget.setParameters
        self.getAdditionalRecoOptions = self._mainWidget.getAdditionalRecoOptions
        self.setAdditionalRecoOptions = self._mainWidget.setAdditionalRecoOptions
        self.getAdditionalPreprocessOptions = self._mainWidget.getAdditionalPreprocessOptions
        self.setAdditionalPreprocessOptions = self._mainWidget.setAdditionalPreprocessOptions
        self.removeOutputDir = self._mainWidget.removeOutputDir
        self.setRemoveOutputDir = self._mainWidget.setRemoveOutputDir

        # menu
        style = qt.QApplication.style()
        saveIcon = style.standardIcon(qt.QStyle.SP_DialogSaveButton)
        self._saveAction = qt.QAction(saveIcon, 'save', self)
        self._saveAction.setToolTip('save configuration')
        self._saveAction.setShortcut(qt.QKeySequence.Save)
        self.menuBar().addAction(self._saveAction)
        self._saveAction.triggered.connect(self.save)

        loadIcon = style.standardIcon(qt.QStyle.SP_DialogOpenButton)
        self._loadAction = qt.QAction(loadIcon, 'load', self)
        self._loadAction.setToolTip('load configuration')
        self._loadAction.setShortcut(qt.QKeySequence.Open)
        self.menuBar().addAction(self._loadAction)
        self._loadAction.triggered.connect(self.load)


class TofuWidget(_TofuOptionLoader, qt.QWidget):
    _OPT_RECO_SECTION_ID = "options_reco"
    """Variable used to define the section name when saving the configuration
    into dict dump"""
    _OPT_PREPROCESS_SECTION_ID = "options_preprocess"

    _TOMWER_CONFIG = "tomwer_configuration"
    "tomwer processing configuration"

    def __init__(self, parent):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QVBoxLayout())
        self._scanTypeWidget = ScanTypeWidget(parent=self)
        self.layout().addWidget(self._scanTypeWidget)

        self._tabs = _TofuTab(parent=self)
        self.layout().addWidget(self._tabs)

        _TofuOptionLoader.__init__(self,
                                   options={
            'z-parameter': _getterSetter(getter=self._scanTypeWidget.getScanType,
                                         setter=self._scanTypeWidget.setScanType)},
                                   childs=[self._tabs,])

        # expose API
        self.getAdditionalRecoOptions = self._tabs.getAdditionalRecoOptions
        self.getAdditionalPreprocessOptions = self._tabs.getAdditionalPreprocessOptions
        self.removeOutputDir = self._tabs._outputWidget._outputWidget.removeIfExist
        self.setRemoveOutputDir = self._tabs._outputWidget._outputWidget.setRemoveIfExist

        # connect signals
        self._scanTypeWidget._scanType.currentIndexChanged.connect(self._scanTypeChanged)
        self._tabs._outputWidget._volumeAngleGrp._grpPlane.sigPlaneChanged.connect(self._scanTypeChanged)
        # update display according to the current scan type
        self._scanTypeChanged()

    def _scanTypeChanged(self, *args, **kwargs):
        scan_type = self._scanTypeWidget._scanType.currentText()
        x_center = 0
        # change step size type
        if scan_type == 'slice stack':
            _type = '(pixel)'
            # z_center = self._tabs._inputWidget._centeringWidget.getZCenter()
            # _range = (z_center - settings.SLICE_STACK_RANGE_HS,
            #           z_center + settings.SLICE_STACK_RANGE_HS)
            _range = -settings.SLICE_STACK_RANGE_HS, settings.SLICE_STACK_RANGE_HS
            _step_size = settings.SLICE_STACK_STEP_SIZE
            _nb_slices = settings.SLICE_STACK_NB_SLICES
        elif scan_type == 'rotation center':
            _type = '(pixel)'
            x_center = self._tabs._inputWidget.centeringWidget.getXCenter()
            _range = (x_center - settings.ROT_CENTER_RANGE_HS,
                      x_center + settings.ROT_CENTER_RANGE_HS)
            _step_size = settings.ROT_CENTER_STEP_SIZE
            _nb_slices = settings.ROT_CENTER_NB_SLICES
        elif scan_type == 'lamino angle':
            _type = '(degree)'
            lamino_angle = self._tabs._inputWidget.rotationAngle.getLaminoAngle()
            x_center = lamino_angle
            _range = (lamino_angle - settings.LAMINO_ANG_RANGE_HS,
                      lamino_angle + settings.LAMINO_ANG_RANGE_HS)
            _step_size = settings.LAMINO_ANG_STEP_SIZE
            _nb_slices = settings.LAMINO_ANG_NB_SLICES
        elif scan_type == PSI_CHAR + ' angle':
            _type = '(degree)'
            psi_angle = self._tabs._inputWidget.rotationAngle.getPsiAngle()
            x_center = psi_angle
            # _range = (psi_angle - settings.PSI_ANG_RANGE_HS,
            #           psi_angle + settings.PSI_ANG_RANGE_HS)
            _range = -settings.PSI_ANG_RANGE_HS, settings.PSI_ANG_RANGE_HS
            _step_size = settings.PSI_ANG_STEP_SIZE
            _nb_slices = settings.PSI_ANG_NB_SLICES
        else:
            raise NotImplementedError('')

        _stepSizeAndRangeWidget = self._tabs._outputWidget._stepSizeAndRange
        _stepSizeAndRangeWidget._setStepSizeType(_type)
        _stepSizeAndRangeWidget._z_label.setVisible(scan_type == 'slice stack')
        _stepSizeAndRangeWidget._zSB.setVisible(scan_type == 'slice stack')
        # change step range and size
        _stepSizeAndRangeWidget.setRegion(region=(_range[0], _range[1], _step_size))
        _stepSizeAndRangeWidget._setXCenter(x_center)
        _stepSizeAndRangeWidget.setNCut(_nb_slices)
        if scan_type == PSI_CHAR + ' angle':
            scan_type = 'psi angle'
        self._tabs._outputWidget._setScanType(scan_type)

    def load(self):
        dialog = qt.QFileDialog(self)
        dialog.setNameFilters(["ini (*.ini)"])
        dialog.setFileMode(qt.QFileDialog.ExistingFile)

        if not dialog.exec_():
            dialog.close()
            return
        else:
            self.loadFile(dialog.selectedFiles()[0])

    def loadFile(self, _file):
        _logger.info(('loading', _file))
        try:
            conf = dictdump.load(_file)
        except Exception as error:
            _logger.warning("Fail to load configuration from ", _file, ". Reason is", error)
        else:
            if self._OPT_RECO_SECTION_ID not in conf:
                _logger.warning('Dict load is not recognized. Fail to load configuration')
                reco_opts = {}
            else:
                reco_opts = conf[self._OPT_RECO_SECTION_ID]
            if self._OPT_PREPROCESS_SECTION_ID not in conf:
                _logger.warning(
                    'Dict load is not recognized. Fail to load configuration')
                preprocess_opts = {}
            else:
                preprocess_opts = conf[self._OPT_PREPROCESS_SECTION_ID]
            if self._TOMWER_CONFIG not in conf:
                _logger.warning(
                    'Dict load is not recognized. Fail to load configuration')
                tomwer_config = {}
            else:
                tomwer_config = conf[self._TOMWER_CONFIG]
            self.setConfigFromOptionsDict(reco_opts=reco_opts,
                                          preprocess_opts=preprocess_opts,
                                          tomwer_config=tomwer_config)

    def save(self):
        dialog = qt.QFileDialog(self)
        dialog.setNameFilters(["ini (*.ini)"])
        dialog.setAcceptMode(qt.QFileDialog.AcceptSave)
        dialog.setFileMode(qt.QFileDialog.AnyFile)

        if not dialog.exec_():
            dialog.close()
            return
        else:
            self.saveTo(dialog.selectedFiles()[0])

    def saveTo(self, _file):
        """Save the parameters used for reconstruction into an ini file"""
        if _file.lower().endswith('.ini') is False:
            _file = _file + '.ini'
        _logger.info(('saving to', _file))
        fixedOptions = self.getParameters()
        additionalRecoOptions = self.getAdditionalRecoOptions()
        additionalPreprocessOptions = self.getAdditionalPreprocessOptions()
        pre_proc_ffc = fixedOptions['ffc-when']
        if type(pre_proc_ffc) is str:
            pre_proc_ffc = getattr(FFCWhen, pre_proc_ffc)
        assert isinstance(pre_proc_ffc, FFCWhen)

        def store(options, key, ddict):
            optionsList = _retrieve_opts_recons_cmd(scan_id=None,
                                                    recons_param=fixedOptions,
                                                    additional_opts=options,
                                                    pre_proc_ffc=pre_proc_ffc is FFCWhen.preprocessing)
            optionsList = optionsList.split('--')
            optionsDict = {}
            for opt in optionsList:
                if opt == ' ':
                    continue
                _opt = opt.lstrip(' ')
                if '=' in _opt:
                    optName, value = _opt.split('=', 1)
                    value = value.replace('=', '')
                elif ' ' in _opt:
                    optName, value = _opt.split(' ', 1)
                    value = value.replace(' ', '')
                else:
                    optName = _opt
                    value = ''
                optName = optName.rstrip(' ')
                optionsDict[optName] = value
            ddict[key] = optionsDict

        ddict = {}
        store(options=additionalRecoOptions, key=self._OPT_RECO_SECTION_ID,
              ddict=ddict)
        store(options=additionalPreprocessOptions, key=self._OPT_PREPROCESS_SECTION_ID,
              ddict=ddict)
        config_dict = {
            'rm-tif': fixedOptions['rm-tif'],
            'ffc-when': fixedOptions['ffc-when'],
            'half-acquisition': fixedOptions['half-acquisition'],
            'blend': fixedOptions['blend'],
            'adjust-mean': fixedOptions['adjust-mean'],
            'z': fixedOptions['z'],
        }
        ddict[self._TOMWER_CONFIG] = config_dict
        dictdump.dicttoini(ddict=ddict, inifile=_file)

    def setConfigFromOptionsDict(self, reco_opts, preprocess_opts,
                                 tomwer_config):
        """
        extract from the command line the list of option names and their
        values if any to store them

        :param list or tuple options:
        """
        self._tabs._expertWidget.resetAdditionalRecoOptions()
        self._tabs._expertWidget.resetAdditionalPreprocessOptions()
        add_options = self.setParameters(reco_opts)
        for option, optValue in add_options.items():
            self._tabs._expertWidget.addAdditionalRecoOption(option=option,
                                                             value=optValue)
        add_options = self.setParameters(preprocess_opts)
        for option, optValue in add_options.items():
            self._tabs._expertWidget.addAdditionalPreprocessOption(option=option,
                                                                   value=optValue)
        if 'rm-tif' in tomwer_config:
            self._tabs._outputWidget.setRemoveTiff(tomwer_config['rm-tif'])
        if 'half-acquisition' in tomwer_config:
            self._tabs._inputWidget.setHalfAcquisition(tomwer_config['half-acquisition'])
        if 'ffc-when' in tomwer_config:
            self._tabs._inputWidget.setWhenApplyFFC(tomwer_config['ffc-when'])
        if 'blend' in tomwer_config:
            self._tabs._inputWidget.setBlend(tomwer_config['blend'])
        if 'adjust-mean' in tomwer_config:
            self._tabs._inputWidget.setAdjustMean(tomwer_config['adjust-mean'])
        if 'z' in tomwer_config:
            self._tabs._outputWidget.setZ(tomwer_config['z'])

    def setAdditionalRecoOptions(self, options):
        assert isinstance(options, str)
        self._tabs._expertWidget.resetAdditionalRecoOptions()

    def setAdditionalPreprocessOptions(self, options):
        assert isinstance(options, str)
        self._tabs._expertWidget.resetAdditionalPreprocessOptions()

    def loadFromScan(self, scanID):
        self._tabs.loadFromScan(scanID)
        self._scanTypeChanged()


class _TofuTab(_TofuOptionLoader, qt.QTabWidget):
    """
    Tab containing the different editor for the tofu reconstruction parameter
    """

    def __init__(self, parent):
        qt.QTabWidget.__init__(self, parent)

        self._outputWidget = OutputTofuWidget(parent=self)
        icon = icons.getQIcon('output')
        self.addTab(self._outputWidget, icon, 'Output definition')
        self._inputWidget = InputProjectionsScrollArea(parent=self)
        icon = icons.getQIcon('input')
        self.addTab(self._inputWidget, icon, 'Projections')
        self._expertWidget = ExpertTofuWidget(parent=self)
        icon = icons.getQIcon('parameters')
        self.addTab(self._expertWidget, icon, 'Expert')
        # TODO: warning, _expertWidget should always set as last tab
        _TofuOptionLoader.__init__(self, options=[], childs=[self._outputWidget,
                                                             self._inputWidget,
                                                             self._expertWidget])

        # connect signal/SLOT
        self._expertWidget._gpuGrp.gpuOptionChanged.connect(self.gpuOptionUpdated)

    def loadFromScan(self, scanID):
        for widget in (self._outputWidget, self._inputWidget,
                       self._expertWidget):
            widget.loadFromScan(scanID=scanID)

    def getAdditionalRecoOptions(self):
        return self._expertWidget.getAdditionalRecoOptions()

    def getAdditionalPreprocessOptions(self):
        return self._expertWidget.getAdditionalPreprocessOptions()

    def gpuOptionUpdated(self):
        highLimit = self._expertWidget.getHighLimit()
        self._outputWidget.setHighLimit(highLimit)


class ScanTypeWidget(_TofuOptionLoader, qt.QWidget):
    """
    Simple widget to define the type of reconstruction we want
    """
    def __init__(self, parent):
        qt.QWidget.__init__(self, parent)
        _TofuOptionLoader.__init__(self, options={"scan_type": self.setScanType})

        self.setLayout(qt.QHBoxLayout())
        self.layout().addWidget(qt.QLabel('scan type:', parent=self))
        self._scanType = qt.QComboBox(parent=self)
        for _type in SCAN_TYPES:
            self._scanType.addItem(_type)
        self.layout().addWidget(self._scanType)

    def setScanType(self, scan_type):
        assert scan_type in SCAN_TYPES_I
        index = self._scanType.findText(SCAN_TYPES_I[scan_type])
        assert index >= 0
        self._scanType.setCurrentIndex(index)

    def getScanType(self):
        return SCAN_TYPES[self._scanType.currentText()]



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "30/09/2019"


import unittest
import tempfile
import os
import shutil
import glob
from collections import namedtuple
from tomwer.test.utils import UtilsTest
from tomwer.synctools.utils.scanstages import ScanStages
from tomwer.core.scan.scanfactory import ScanFactory
from tomwer.test.utils import skip_gui_test

if skip_gui_test() is False:
    from tomwer.synctools.rsyncmanager import RSyncManager


_stage_desc = namedtuple("_stage_desc",
                         ["n_par_file", "n_edf_file", "n_info_file",
                          "n_xml_file", "n_vol", "folder_should_exists"])  # noqa


@unittest.skipIf(RSyncManager().has_rsync() is False, "Rsync is missing")
@unittest.skipIf(skip_gui_test(), reason='skip gui test')
class TestScanStagesEDF(unittest.TestCase):
    """
    test the ScanStages class for edf scans
    """
    def setUp(self) -> None:
        self.output_dir = tempfile.mkdtemp()
        dataset = 'D2_H2_T2_h_'
        self.data_test_dir = UtilsTest.getEDFDataset(dataset)
        self.target_dir = os.path.join(self.output_dir, dataset)
        self.scan_stages = ScanStages(scan=ScanFactory.create_scan_object(self.data_test_dir))

    def tearDown(self) -> None:
        shutil.rmtree(self.output_dir)

    def testRSyncUntil(self):
        """test the rsync_until function"""
        tomo_n = 3605
        stage_res = {
            ScanStages.AcquisitionStage.ACQUI_NOT_STARTED: _stage_desc(n_par_file=0,
                                                                       n_edf_file=0,
                                                                       n_info_file=0,
                                                                       n_xml_file=0,
                                                                       n_vol=0,
                                                                       folder_should_exists=False),
            ScanStages.AcquisitionStage.ACQUI_STARTED: _stage_desc(n_par_file=0,
                                                                   n_edf_file=0,
                                                                   n_info_file=1,
                                                                   n_xml_file=0,
                                                                   n_vol=0,
                                                                   folder_should_exists=True),
            ScanStages.AcquisitionStage.ACQUI_ON_GOING: _stage_desc(n_par_file=0,
                                                                    n_edf_file=tomo_n // 2,
                                                                    n_info_file=1,
                                                                    n_xml_file=0,
                                                                    n_vol=0,
                                                                    folder_should_exists=True),
            ScanStages.AcquisitionStage.ACQUI_ENDED: _stage_desc(n_par_file=0,
                                                                 n_edf_file=tomo_n,
                                                                 n_info_file=1,
                                                                 n_xml_file=1,
                                                                 n_vol=0,
                                                                 folder_should_exists=True),
            ScanStages.AcquisitionStage.RECONSTRUCTION_ADDED: _stage_desc(n_par_file=6,
                                                                          n_edf_file=tomo_n,
                                                                          n_info_file=2,
                                                                          n_xml_file=2,
                                                                          n_vol=1,
                                                                          folder_should_exists=True),
            ScanStages.AcquisitionStage.COMPLETE: _stage_desc(n_par_file=6,
                                                              n_edf_file=3719, # there is several ref, darks...
                                                              n_info_file=2,
                                                              n_xml_file=2,
                                                              n_vol=1,
                                                              folder_should_exists=True),
        }
        for stage, th_results in stage_res.items():
            with self.subTest(stage=stage):
                self.scan_stages.rsync_until(stage=stage,
                                             dest_dir=self.output_dir)
                self.assertEqual(os.path.exists(self.target_dir),
                                 th_results.folder_should_exists)
                self.assertEqual(len(glob.glob(os.path.join(self.target_dir, '*.par'))),
                                 th_results.n_par_file)
                self.assertEqual(len(glob.glob(os.path.join(self.target_dir, '*.edf'))),
                                 th_results.n_edf_file)
                self.assertEqual(len(glob.glob(os.path.join(self.target_dir, '*.xml'))),
                                 th_results.n_xml_file)
                self.assertEqual(len(glob.glob(os.path.join(self.target_dir, '*.vol'))),
                                 th_results.n_vol)
                self.assertEqual(len(glob.glob(os.path.join(self.target_dir, '*.info'))),
                                 th_results.n_info_file)


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestScanStagesEDF, ):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == '__main__':
    unittest.main(defaultTest="suite")



# coding: utf-8
#/*##########################################################################
# Copyright (C) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "05/02/2020"


import os
from silx.gui import qt
from tomwer.test.utils import skip_gui_test
from tomwer.gui.reconstruction.nabu.nabuflow import NabuFlowControl
from tomwer.gui.reconstruction.nabu.nabuconfig.preprocessing import _NabuPreProcessingConfig
from tomwer.gui.reconstruction.nabu.nabuconfig.phase import _NabuPhaseConfig
from tomwer.gui.reconstruction.nabu.nabuconfig.reconstruction import _NabuReconstructionConfig
from tomwer.gui.reconstruction.nabu.nabuconfig.output import _NabuOutputConfig

from silx.gui.utils.testutils import TestCaseQt
import unittest


class ProcessClass:
    """Simple class for unit tests"""
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name


@unittest.skipIf(skip_gui_test(), reason='skip gui test')
class TestNabuFlow(TestCaseQt):
    def setUp(self) -> None:
        TestCaseQt.setUp(self)
        self.nabuWidget = NabuFlowControl(parent=None, direction='vertical')

    def tearDown(self):
        self.nabuWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.nabuWidget.close()
        del self.nabuWidget

    def testFlow1(self):
        style = qt.QApplication.style()
        icon_1 = style.standardIcon(qt.QStyle.SP_DialogApplyButton)
        icon_2 = style.standardIcon(qt.QStyle.SP_FileLinkIcon)
        icon_3 = style.standardIcon(qt.QStyle.SP_ArrowLeft)
        icon_4 = style.standardIcon(qt.QStyle.SP_ArrowRight)
        icon_5 = style.standardIcon(qt.QStyle.SP_BrowserStop)

        preprocess = 'reading files', ProcessClass(name='other preprocessing')
        preprocess_icons = None, icon_1
        self.nabuWidget.setPreProcessing(processes=preprocess,
                                         icons=preprocess_icons)

        processes = 'processing 1', ProcessClass(
            'in between processing'), 'other processing'
        processes_icons = icon_2, None, icon_3
        self.nabuWidget.setProcessing(processes=processes,
                                      icons=processes_icons)

        postprocess = 'post processing', ProcessClass('writing result')
        postprocess_icons = icon_4, icon_5
        self.nabuWidget.setPostProcessing(processes=postprocess,
                                          icons=postprocess_icons)


@unittest.skipIf(skip_gui_test(), reason='skip gui test')
class TestNabuPreProcConfig(TestCaseQt):
    def setUp(self):
        TestCaseQt.setUp(self)
        self.nabuWidget = _NabuPreProcessingConfig(parent=None)

    def tearDown(self):
        self.nabuWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.nabuWidget.close()
        del self.nabuWidget
        self.qapp.processEvents()
        TestCaseQt.tearDown(self)

    def testGetInitialConfiguration(self):
        """Test that the get configuration is working"""
        ini_conf = {
            'flatfield_enabled': 1,
            'double_flatfield_enabled': 0,
            'dff_sigma': 0.0,
            'ccd_filter_enabled': 0,
            'ccd_filter_threshold': 0.04,
            'log_min_clip': 1e-6,
            'log_max_clip': 10.0,
            'take_logarithm': True,
        }
        self.assertEqual(self.nabuWidget.getConfiguration(), ini_conf)

    def testSetConfiguration(self):
        """Test that the set configuration is working"""
        conf = {
            'flatfield_enabled': 0,
            'double_flatfield_enabled': 1,
            'dff_sigma': 2.0,
            'ccd_filter_enabled': 1,
            'ccd_filter_threshold': 0.98,
            'log_min_clip': 1e-3,
            'log_max_clip': 250.0,
            'take_logarithm': False,
        }
        self.nabuWidget.setConfiguration(conf=conf)
        self.assertEqual(self.nabuWidget.getConfiguration(), conf)


@unittest.skipIf(skip_gui_test(), reason='skip gui test')
class TestNabuPhaseConfig(TestCaseQt):
    def setUp(self):
        TestCaseQt.setUp(self)
        self.nabuWidget = _NabuPhaseConfig(parent=None)

    def tearDown(self):
        self.nabuWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.nabuWidget.close()
        del self.nabuWidget
        TestCaseQt.tearDown(self)

    def testGetInitialConfiguration(self):
        """Test that the get configuration is working"""
        ini_conf = {
            'method': 'Paganin',
            'delta_beta': '100.0',
            'margin': 50,
            'padding_type': 'edge',
            'unsharp_coeff': 0,
            'unsharp_sigma': 0,
        }
        self.assertEqual(self.nabuWidget.getConfiguration(), ini_conf)

    def testSetConfiguration(self):
        """Test that the set configuration is working"""
        conf = {
            'method': 'Paganin',
            'delta_beta': '200.0',
            'margin': 20,
            'padding_type': 'zeros',
            'unsharp_coeff': 3.6,
            'unsharp_sigma': 2.1,
        }
        self.nabuWidget.setConfiguration(conf)
        self.nabuWidget.show()
        # check visibility of some widgets
        unsharp_widget = self.nabuWidget._unsharpOpts
        self.assertEqual(unsharp_widget._unsharpCoeffQLE.text(),
                        '3.6')
        self.assertTrue(unsharp_widget._unsharpCoeffCB.isChecked())

        self.assertEqual(unsharp_widget._unsharpSigmaQLE.text(),
                        '2.1')
        self.assertTrue(unsharp_widget._unsharpSigmaCB.isChecked())
        paganin_widget = self.nabuWidget._paganinOpts
        self.assertEqual(paganin_widget._deltaBetaQLE.text(), '200.0')
        self.assertEqual(paganin_widget._paddingTypeCB.currentText(),
                         'zeros')

        # check the generated configuration
        self.assertEqual(self.nabuWidget.getConfiguration(), conf)


@unittest.skipIf(skip_gui_test(), reason='skip gui test')
class TestNabuReconstructionConfig(TestCaseQt):
    def setUp(self):
        TestCaseQt.setUp(self)
        self.nabuWidget = _NabuReconstructionConfig(parent=None)

    def tearDown(self):
        self.nabuWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.nabuWidget.close()
        del self.nabuWidget
        TestCaseQt.tearDown(self)

    def testGetInitialConfiguration(self):
        """Test that the get configuration is working"""
        ini_conf = {
            'method': "FBP",
            'angles_file': '',
            'axis_correction_file': '',
            'angle_offset': 0.,
            'fbp_filter_type': 'ramlak',
            'padding_type': 'zeros',
            'start_x': 0,
            'end_x': -1,
            'start_y': 0,
            'end_y': -1,
            'start_z': 0,
            'end_z': -1,
            'iterations': 200,
            'optim_algorithm': "chambolle-pock",
            'weight_tv': 1.0e-2,
            'preconditioning_filter': 1,
            'rotation_axis_position': '',
            'positivity_constraint': 1,
            'translation_movements_file': '',
        }
        self.assertEqual(self.nabuWidget.getConfiguration(), ini_conf)

    def testSetConfiguration(self):
        """Test that the set configuration is working"""
        ini_conf = {
            'method': "FBP",
            'angles_file': '',
            'axis_correction_file': '',
            'angle_offset': 12.5,
            'fbp_filter_type': 'none',
            'padding_type': 'edges',
            'start_x': 0,
            'end_x': 23,
            'start_y': 12,
            'end_y': 56,
            'start_z': 560,
            'end_z': -1,
            'iterations': 20,
            'optim_algorithm': "chambolle-pock",
            'weight_tv': 1.5e-2,
            'preconditioning_filter': 0,
            'rotation_axis_position': '',
            'positivity_constraint': 0,
            'translation_movements_file': 'my_file.csv',

        }
        self.nabuWidget.setConfiguration(ini_conf)
        qt.QApplication.processEvents()
        self.nabuWidget.show()

        # check visibility of some widgets
        self.assertEqual(self.nabuWidget._angleOffsetQDSB.value(), 12.5)
        subRegionWidget = self.nabuWidget._subRegionSelector
        self.assertFalse(subRegionWidget._xSubRegion._minCB.isChecked())
        self.assertFalse(subRegionWidget._xSubRegion._minQLE.isEnabled())
        self.assertTrue(subRegionWidget._xSubRegion._maxQLE.isEnabled())
        self.assertTrue(subRegionWidget._ySubRegion._minQLE.isEnabled())
        self.assertTrue(subRegionWidget._ySubRegion._maxQLE.isEnabled())
        self.assertTrue(subRegionWidget._zSubRegion._minQLE.isEnabled())
        self.assertFalse(subRegionWidget._zSubRegion._maxQLE.isEnabled())
        self.assertFalse(self.nabuWidget._preconditioningFilter.isChecked())

        # check the generated configuration
        self.assertEqual(self.nabuWidget.getConfiguration(), ini_conf)


class TestNabuOutputConfig(TestCaseQt):
    """Test the output configuration interface"""
    def setUp(self):
        TestCaseQt.setUp(self)
        self.nabuWidget = _NabuOutputConfig(parent=None)

    def tearDown(self):
        self.nabuWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.nabuWidget.close()
        self.nabuWidget = None
        self.qapp.processEvents()
        TestCaseQt.tearDown(self)

    def testGetConfiguration(self):
        ini_conf = {
            'file_format': "hdf5",
            'location': '',
        }
        self.assertEqual(self.nabuWidget.getConfiguration(), ini_conf)

    def testSetConfiguration(self):
        conf = {
            'file_format': "npz",
            'location': os.sep.join(('tmp', 'my_output')),
        }
        self.nabuWidget.setConfiguration(conf)
        self.nabuWidget.show()
        self.qapp.processEvents()
        # check some widget visibility
        self.assertTrue(self.nabuWidget._outputDirQLE.isVisible())
        self.assertFalse(self.nabuWidget._defaultOutput.isChecked())

        self.assertEqual(self.nabuWidget.getConfiguration(), conf)


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestNabuPreProcConfig, TestNabuPhaseConfig, TestNabuFlow,
               TestNabuReconstructionConfig, TestNabuOutputConfig):
        test_suite.addTest(
            unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == '__main__':
    unittest.main(defaultTest="suite")




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sys
import argparse
import glob
import os
import shutil
from tomwer.core.scan.scanfactory import ScanFactory
from tomwer.synctools.utils.scanstages import ScanStages


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def getinputinfo():
    return "tomwer rsync [source_dir] [dest_dir] [options]"


def main(argv):
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        'source_dir',
        help='folder containing the acquisition to copy.'
             'To copy /dir_path/scan_dir into /dir_path2/scan_dir source-dir '
             'should be /dir_path/scan_dir')
    parser.add_argument(
        'dest_dir',
        help='destination folder. If doesn\'t exists will create it.'
             'Will synchronize'
             'If exists and contains some files, those won\'t be remove'
             'except if the --fresh option is activated'
             'To copy /dir_path/scan_dir into /dir_path2/scan_dir dest-dir '
             'should be /dir_path2')
    parser.add_argument(
        '--advancement',
        help='An acquisition contains several steps (initialization, '
             'acquisition, complete...). You can ask to synchronize file until'
             ' a specific stage. \n'
             'For edf for example if you specify the stage `on_going` no .info'
             ' while be created and the targeted directory won\'t contain '
             'any \'slice\' files. \n'
             'Default value is `complete`'
             'Valid advancement values are: %s' % str(ScanStages.AcquisitionStage.get_command_names()))
    parser.add_argument(
        '--fresh',
        help='first file to be used for computing axis',
        action='store_true',
        default=False)
    parser.add_argument(
        '--force', '-f',
        help='force folder removal',
        action='store_true',
        default=False)
    parser.add_argument(
        '--rm-final-xml',
        help='util option: remove the .xml created at the end of some .edf'
             'acquisition (which notify the end of the acquisition). \n'
             'Warning: this is an exclusive option. So it cannot be associated'
             'with any other option.',
        action='store_true',
        default=False)

    options = parser.parse_args(argv[1:])

    if (options.rm_final_xml is True and
            ((options.advancement is not None) or (options.fresh is True))):
        logger.error('--rm-final-xml is an `exclusive`option. It cannot be'
                     'associated with an other option')
        return

    if options.advancement is None and options.rm_final_xml is False:
        options.advancement = 'complete'

    for scan_path in glob.glob(options.source_dir):
        try:
            scan = ScanFactory.create_scan_object(scan_path=scan_path)
        except ValueError:
            logger.error(scan_path + ' is not containing a scan')
            continue

        dest_dir = os.path.join(options.dest_dir, os.path.basename(options.source_dir))

        # fresh option treatment
        def refresh_dir(dir_):
            logger.info('recreate', dir_)
            shutil.rmtree(dir_)
            os.mkdir(dir_)

        if options.rm_final_xml is True:
            final_xml = os.path.join(dest_dir, os.path.basename(dest_dir) + '.xml')
            if os.path.exists(final_xml):
                os.remove(final_xml)
            # if the xml option is created then other options are ignore
            continue

        if options.fresh and os.path.exists(dest_dir):
            if options.force is False:
                conf = input("do you want to remove %s (yes/y) ?" % dest_dir)
                if conf.lower() in ('yes', 'y'):
                    refresh_dir(dir_=dest_dir)
            else:
                refresh_dir(dir_=dest_dir)

        scan_stages = ScanStages(scan=scan)
        stage = ScanStages.AcquisitionStage.from_command_name(options.advancement)
        scan_stages.rsync_until(stage=stage,
                                dest_dir=dest_dir)


if __name__ == '__main__':
    main(sys.argv)



#/*##########################################################################
# Copyright (C) 2016-2017 European Synchrotron Radiation Facility
#
# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
# the ESRF by the Software group.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/

__author__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/08/2018"


import functools
import logging
import os
import h5py

from silx.gui import qt
from silx.io.url import DataUrl

from tomwer.core.utils import ftseriesutils
from tomwer.core.scan.scanbase import TomwerScanBase
from tomwer.synctools.imageloaderthread import ImageLoaderThread

logger = logging.getLogger(__name__)


class _Image(qt.QObject):
    def __init__(self, data):
        qt.QObject.__init__(self)
        self.data = data


class ImageFromFile(_Image):
    """
    Define an Image with a status
    """
    LOADING_STATUS = {'loading': 0, 'loaded': 1, 'not loaded': 2}
    sigLoaded = qt.Signal(str)
    """Signal emitted when the file is loaded. Parameter is the file url"""

    def __init__(self, _file=None, index=None, data=None, _load=False, url=None):
        _Image.__init__(self, data)
        if url is not None:
            assert _file is None and index is None
            self.url = url
        else:
            assert isinstance(_file, str)
            data_path = None
            if _file.lower().endswith('.edf'):
                scheme = 'fabio'
            elif _file.lower().endswith(('.npy', '.npz')):
                scheme = 'numpy'
            else:
                def get_nabu_entry():
                    try:
                       with h5py.File(_file, mode='r') as h5s:
                           for node in h5s:
                               if 'reconstruction' in h5s[node]:
                                   return '/'.join((node, 'reconstruction', 'results', 'data'))
                    except Exception as e:
                        logger.error(e)

                data_path = get_nabu_entry() or None
                scheme = 'silx'

            if index is not None:
                data_slice = (index,)
            else:
                data_slice = None
            self.url = DataUrl(file_path=_file, data_slice=data_slice,
                               scheme=scheme, data_path=data_path)

        self._status = 'not loaded' if data is None else 'loaded'
        self.url_path = self.url.path()
        if _load is True:
            self.load()

    def load(self, sync=False):
        """
        Load the data contained in the url.

        :param sync: if True: then wait for the image to be loaded for giving
            back hand
        """
        if self._status == 'loaded' and self.data is not None:
            self.sigLoaded.emit(self.url.path())
        elif self._status == 'loading':
            logger.debug('%s is already loading' % self.url.path())
        else:
            self._status = 'loading'
            self.loaderThread = ImageLoaderThread(url=self.url)

            callback = functools.partial(self._setData, self.url_path,
                                         self.loaderThread.getData)
            self.loaderThread.finished.connect(callback)
            self.loaderThread.start()
            if sync is False:
                self.loaderThread.wait()

    def _setData(self, url_path, dataGetter):
        self.data = dataGetter()
        if self.data is not None:
            if self.data.ndim != 2:
                if self.data.shape[0] == 1:
                    self.data = self.data.reshape(self.data.shape[1:])
                elif self.data.shape[-1] == 1:
                    self.data = self.data.reshape(self.data.shape[0:-1])

        self._status = 'loaded'
        self.sigLoaded.emit(url_path)

    def isLoaded(self):
        return self._status == 'loaded'


class FileWithImage(object):
    """Definition of file which can contain multiple images"""
    def __init__(self, _file):
        self.file = _file

    def getImages(self, _load):
        """

        :param bool _load: if True then launch the load of the image
        :return: the list of images contained in the given file.
        """
        if self.file.endswith('.vol'):
            return self._dealWithVolFile(self.file, _load)
        elif self.file.lower().endswith(TomwerScanBase.VALID_RECONS_EXTENSION):
            return [ImageFromFile(self.file), ]
        else:
            logger.error('only deal with .edf and .vol extension')
            return []

    def _dealWithVolFile(self, _file, _load):
        _imagesFiles = []
        volInfoFile = _file.replace('.vol', '.vol.info')
        if not os.path.exists(volInfoFile):
            mess = "Can't find description file %s associated with raw " \
                   "data file %s " % (volInfoFile, _file)
            logger.warning(mess)
        else:
            shape = ftseriesutils._getShapeForVolFile(volInfoFile)
            if shape is not None:
                for zSlice in range(shape[0]):
                    if _file.lower().endswith('.edf'):
                        scheme = 'fabio'
                    else:
                        scheme = 'tomwer'
                    _imagesFiles.append(
                            ImageFromFile(url=DataUrl(file_path=_file,
                                                       data_slice=(zSlice, ),
                                                       scheme=scheme),
                                          _load=_load))
            return _imagesFiles



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import logging
import os
import signal
import sys

from silx.gui import qt
from tomwer.gui import icons
from tomwer.gui.reconstruction.darkref.darkrefwidget import DarkRefWidget
from tomwer.synctools.ftseries import _QDKRFRP
from tomwer.gui.utils.splashscreen import getMainSplashScreen
from tomwer.core.scan.scanfactory import ScanFactory
from tomwer.core.scan.hdf5scan import HDF5TomoScan

logging.basicConfig()
_logger = logging.getLogger(__name__)


class _DarkRefWidgetRunnable(DarkRefWidget):
    sigScanReady = qt.Signal(str)

    def __init__(self, scan, parent=None):
        self.__scan = scan
        self.__darkref_rp = _QDKRFRP()
        DarkRefWidget.__init__(self, parent=parent,
                               reconsparams=self.__darkref_rp)
        buttonExec = qt.QPushButton('execute', parent=self)
        buttonExec.setAutoDefault(True)
        # needed to be used as an application to return end only when the
        # processing thread is needed
        self._forceSync = True
        self.layout().addWidget(buttonExec)
        buttonExec.pressed.connect(self._process)
        self.setWindowIcon(icons.getQIcon('tomwer'))

    def _process(self):
        self.process(scan=self.__scan)


def getinputinfo():
    return "tomwer darkref [scanDir]"


def sigintHandler(*args):
    """Handler for the SIGINT signal."""
    qt.QApplication.quit()


def main(argv):
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        'scan_path',
        help='Data file to show (h5 file, edf files, spec files)')
    parser.add_argument('--entry',
        help='an entry can be specify in case of hdf5 the master file',
        default=None,
    )
    parser.add_argument(
        '--debug',
        dest="debug",
        action="store_true",
        default=False,
        help='Set logging system in debug mode')

    options = parser.parse_args(argv[1:])

    if options.debug:
        logging.root.setLevel(logging.DEBUG)

    if options.entry is None:
        scan = ScanFactory.create_scan_object(options.scan_path)
    else:
        scan = HDF5TomoScan(scan=options.scan_path, entry=options.entry)


    global app  # QApplication must be global to avoid seg fault on quit
    app = qt.QApplication.instance() or qt.QApplication([])

    qt.QLocale.setDefault(qt.QLocale(qt.QLocale.English))
    qt.QLocale.setDefault(qt.QLocale.c())
    signal.signal(signal.SIGINT, sigintHandler)
    sys.excepthook = qt.exceptionHandler

    timer = qt.QTimer()
    timer.start(500)
    # Application have to wake up Python interpreter, else SIGINT is not
    # catched
    timer.timeout.connect(lambda: None)

    splash = getMainSplashScreen()

    widget = _DarkRefWidgetRunnable(scan)
    splash.finish(widget)
    widget.show()
    app.exec_()


if __name__ == '__main__':
    main(sys.argv)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "11/03/2020"

from silx.gui import qt
from Orange.widgets import widget, gui
from tomwer.core.utils import logconfig
from tomwer.gui.control.datalist import HDF5DataListDialog
from tomwer.core.scan.hdf5scan import HDF5TomoScan
from tomwer.web.client import OWClient
from Orange.widgets.settings import Setting
from Orange.widgets.widget import Output
from tomwer.core.scan.scanbase import TomwerScanBase
from nxtomomill import converter as nxtomomill_converter
import os
import logging
logger = logging.getLogger(__name__)


def _nxtomomill_input_callback(entry, desc):
    """
    callback provided to nxtomomill if an entry is missing.
    The goal is to ask the user the missing informations

    :param entry:
    :param desc:
    :return:
    """
    text, ok_pressed = qt.QInputDialog.getText(None, 'fill missing entry {}'.format(entry),
                                               desc)

    if ok_pressed:
        return text
    else:
        return None


class NXTomomillOW(widget.OWWidget, OWClient):
    """
    Widget to allow user to pick some bliss files and that will convert them
    to HDF5scan.
    """
    name = "nxtomomill"
    id = "orange.widgets.tomwer.control.NXTomomillOW.NXTomomillOW"
    description = "Read a bliss .h5 file and extract from it all possible" \
                  "NxTomo. When validated create a TomwerBaseScan for each " \
                  "file and entry"
    icon = "icons/nxtomomill.svg"
    priority = 120
    category = "esrfWidgets"
    keywords = ["hdf5", "nexus", "tomwer", 'file', "convert", "NXTomo",
                "tomography"]

    want_main_area = True
    resizing_enabled = True
    compress_signal = False

    _scans = Setting(list())

    class Outputs:
        data_out = Output(name='data',
                          type=TomwerScanBase,
                          doc='one scan to be process')

    def __init__(self, parent=None):
        widget.OWWidget.__init__(self, parent)
        OWClient.__init__(self, logger)
        self.widget = HDF5DataListDialog(parent=self)
        self.widget.setScanIDs(self._scans)
        layout = gui.vBox(self.mainArea, self.name).layout()
        layout.addWidget(self.widget)
        self.widget._sendButton.clicked.connect(self._sendList)
        self.__request_input = True
        # do we ask the user for input if missing

        # expose API
        self.add = self.widget.add
        self.n_scan = self.widget.n_scan

        # alias used for the 'simple workflow' for now
        self.start = self._sendList

    @property
    def request_input(self):
        return self.__request_input

    @request_input.setter
    def request_input(self, request):
        self.__request_input = request

    def _sendList(self):
        """Send a signal for each list to the next widget"""
        for bliss_url in self.widget.datalist.items:
            entry, file_path = bliss_url.split('@')
            file_dir = os.path.dirname(file_path)
            file_name = os.path.basename(file_path)
            if '.' in file_name:
                file_name = ''.join(file_name.split('.')[:-1])

            entry_for_file_name = entry.lstrip('/')
            entry_for_file_name = entry_for_file_name.replace('/', '_')
            output_file_name = '_'.join((os.path.splitext(file_name)[0],
                                         entry_for_file_name + '.nx'))
            output_file_path = os.path.join(file_dir, output_file_name)

            if os.path.exists(output_file_path):
                if not self._userAgreeForOverwrite(output_file_path):
                    continue
            try:
                conv = nxtomomill_converter.h5_to_nx(input_file_path=file_path,
                                                     output_file=output_file_path,
                                                     entries=(entry,),
                                                     single_file=True,
                                                     ask_before_overwrite=False,
                                                     request_input=self.request_input,
                                                     file_extension='.nx',
                                                     input_callback=_nxtomomill_input_callback)
            except Exception as e:
                logger.error(
                    'Fail to convert from bliss file: %s to NXTomo.'
                    'Conversion error is: %s' % (bliss_url, e))
            else:
                if len(conv) != 1:
                    logger.error(
                        'Something went wrong during file conversion. Unable to process the file.')
                else:
                    conv_file, conv_entry = conv[0]
                    scan_converted = HDF5TomoScan(scan=conv_file, entry=conv_entry)
                    mess = "sending one scan %s" % bliss_url
                    logger.debug(mess,
                                 extra={logconfig.DOC_TITLE: self.widget._scheme_title,
                                        logconfig.SCAN_ID: bliss_url})
                    self.Outputs.data_out.send(scan_converted)

    def _userAgreeForOverwrite(self, file_path):
        msg = qt.QMessageBox(self)
        msg.setIcon(qt.QMessageBox.Question)
        types = qt.QMessageBox.Ok | qt.QMessageBox.Cancel
        msg.setStandardButtons(types)
        text = "NXtomomill will overwrite \n %s. Do you agree ?" % file_path
        msg.setText(text)
        return msg.exec_() == qt.QMessageBox.Ok



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2020 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "11/02/2020"


import typing
import os
import functools
from tomwer.utils import docstring
from nabu.utils import PaddingMode
from silx.gui import qt
from tomwer.gui.reconstruction.nabu.nabuconfig import base
from tomwer.gui.utils.inputwidget import SelectionLineEdit
from tomwer.core.utils.char import BETA_CHAR, DELTA_CHAR
from tomwer.core.process.reconstruction.nabu import (_NabuStages,
                                                     _NabuPhaseMethod)
import logging
_logger = logging.getLogger(__name__)


class _NabuPhaseConfig(qt.QWidget, base._NabuStageConfigBase):
    """
    Widget to define the configuration of the nabu preprocessing
    """

    sigConfChanged = qt.Signal(str)
    """Signal emitted when the configuration change. Parameter is the option
    modified
    """

    def __init__(self, parent):
        qt.QWidget.__init__(self, parent, stage=_NabuStages.PHASE)
        base._NabuStageConfigBase.__init__(self, stage=_NabuStages.PHASE)
        self.setLayout(qt.QGridLayout())

        # phase method
        self._methodLabel = qt.QLabel('method', self)
        self.layout().addWidget(self._methodLabel, 1, 0, 1, 1)
        self._methodCB = qt.QComboBox(parent=self)
        for method in _NabuPhaseMethod:
            self._methodCB.addItem(method.value)
        self.layout().addWidget(self._methodCB, 1, 1, 1, 3)
        self.registerWidget(self._methodLabel, 'required')
        self.registerWidget(self._methodCB, 'required')

        # paganin options
        self._paganinOpts = NabuPaganinConfig(parent=self)
        self.layout().addWidget(self._paganinOpts, 2, 0, 3, 3)

        # unsharp options
        self._unsharpOpts = NabuUnsharpConfig(parent=self)
        self.layout().addWidget(self._unsharpOpts, 6, 0, 3, 3)

        # spacer for style
        spacer = qt.QWidget(self)
        spacer.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Expanding)
        self.layout().addWidget(spacer, 200, 1, 1, 1)

        # set up
        item_index = self._methodCB.findText(_NabuPhaseMethod.PAGANIN.value)
        assert item_index >= 0
        self._methodCB.setCurrentIndex(item_index)

        # connect signal / slot
        self._methodCB.currentIndexChanged.connect(self._methodChanged)
        self._paganinOpts.sigConfChanged.connect(self._signalConfChanged)

    def _methodChanged(self, *args, **kwargs):
        self._signalConfChanged('method')

    def _signalConfChanged(self, param):
        self.sigConfChanged.emit(param)

    def getMethod(self) -> _NabuPhaseMethod:
        return _NabuPhaseMethod.from_value(self._methodCB.currentText())

    @docstring(base._NabuStageConfigBase)
    def setConfiguration(self, config) -> None:
        if 'method' in config:
            method = config['method']
            if method in (None, 'none', 'None'):
                pass
            if isinstance(method, _NabuPhaseMethod):
                method = method.value
            index_method = self._methodCB.findText(method)
            if index_method >= 0:
                self._methodCB.setCurrentIndex(index_method)
                self._paganinOpts.setConfiguration(config)
            else:
                _logger.error('unable to find method ' + method)
        self._unsharpOpts.setConfiguration(config)

    @docstring(base._NabuStageConfigBase)
    def getConfiguration(self) -> dict:
        conf_this_widget = {
            'method': self.getMethod().value,
        }
        if self.getMethod() is _NabuPhaseMethod.PAGANIN:
            conf_this_widget.update(self._paganinOpts.getConfiguration())
        conf_this_widget.update(self._unsharpOpts.getConfiguration())
        return conf_this_widget

    def setConfigurationLevel(self, level):
        base._NabuStageConfigBase.setConfigurationLevel(self, level)
        self._unsharpOpts.setConfigurationLevel(level=level)
        self._paganinOpts.setConfigurationLevel(level=level)


class NabuPaganinConfig(qt.QWidget, base._NabuStageConfigBase):
    """Configuration widget dedicated to the paganin options for nabu"""

    sigConfChanged = qt.Signal(str)
    """Signal emitted when the configuration change. Parameter is the option
    modified
    """

    def __init__(self, parent):
        qt.QWidget.__init__(self, parent=parent, stage=_NabuStages.PHASE)
        base._NabuStageConfigBase.__init__(self, stage=_NabuStages.PHASE)
        self.setLayout(qt.QGridLayout())

        # paganin delta / beta
        label = DELTA_CHAR + ' / ' + BETA_CHAR
        self._db_label = qt.QLabel(label, self)
        self.layout().addWidget(self._db_label, 0, 0, 1, 1)
        self._deltaBetaQLE = SelectionLineEdit('100.0', self)
        self.layout().addWidget(self._deltaBetaQLE, 0, 1, 1, 3)
        self.registerWidget(self._db_label, 'required')
        self.registerWidget(self._deltaBetaQLE, 'required')

        # paganin lmicron
        # we skip this paramter, only used for compatibility

        # paganin margin
        self.layout().addWidget(qt.QLabel('margin'), 1, 0, 1, 1)
        self._margeQSB = qt.QSpinBox(self)
        self._margeQSB.setMinimum(0)
        self._margeQSB.setMaximum(9999)
        self._margeQSB.setToolTip("Marge (in pixels) in the Paganin filtering "
                                  "to avoid local tomography artefacts")
        self.layout().addWidget(self._margeQSB, 1, 1, 1, 1)
        self.registerWidget(self._margeQSB, 'optional')

        # paganin padding_type
        self._paddingLabel = qt.QLabel('padding', self)
        self.layout().addWidget(self._paddingLabel, 2, 0, 1, 1)
        self._paddingTypeCB = qt.QComboBox(self)
        for padding_type in PaddingMode:
            self._paddingTypeCB.addItem(padding_type.value)
        self.layout().addWidget(self._paddingTypeCB, 2, 1, 1, 3)
        self.registerWidget(self._paddingLabel, 'advanced')
        self.registerWidget(self._paddingTypeCB, 'advanced')

        # set up
        self._margeQSB.setValue(50)
        item_index = self._paddingTypeCB.findText(PaddingMode.EDGE.value)
        self._paddingTypeCB.setCurrentIndex(item_index)

        # connect signal - slot
        self._deltaBetaQLE.editingFinished.connect(self._paganinDBChanged)
        self._margeQSB.editingFinished.connect(self._paganinMargeChanged)
        self._paddingTypeCB.currentIndexChanged.connect(self._paganinPaddingTypeChanged)

    def _paganinDBChanged(self, *args, **kwargs):
        self._signalConfChanged('delta_beta')

    def _paganinMargeChanged(self, *args, **kwargs):
        self._signalConfChanged('marge')

    def _paganinPaddingTypeChanged(self, *args, **kwargs):
        self._signalConfChanged('padding_type')

    def getPaddingType(self) -> PaddingMode:
        current_text = self._paddingTypeCB.currentText()
        return PaddingMode.from_value(current_text)

    def getMarge(self) -> int:
        return int(self._margeQSB.text())

    def getDeltaBeta(self) -> str:
        return self._deltaBetaQLE.text()

    def getConfiguration(self):
        return {
            'delta_beta': self.getDeltaBeta(),  # this one is not cast because can contain several values
            'margin': self.getMarge(),
            'padding_type': self.getPaddingType().value,
        }

    def setConfiguration(self, conf):
        if 'delta_beta' in conf:
            self._deltaBetaQLE.setText(str(conf['delta_beta']))
        if 'margin' in conf:
            self._margeQSB.setValue(int(conf['margin']))
        if 'padding_type' in conf:
            padding_type = PaddingMode.from_value(conf['padding_type'])
            item_index = self._paddingTypeCB.findText(padding_type.value)
            self._paddingTypeCB.setCurrentIndex(item_index)

    def _signalConfChanged(self, param):
        self.sigConfChanged.emit(param)


class NabuUnsharpConfig(qt.QWidget, base._NabuStageConfigBase):
    """Configuration widget dedicated to the unsharp options for nabu"""

    sigConfChanged = qt.Signal(str)
    """Signal emitted when the configuration change. Parameter is the option
    modified
    """

    def __init__(self, parent):
        qt.QWidget.__init__(self, parent=parent, stage=_NabuStages.PHASE)
        base._NabuStageConfigBase.__init__(self, stage=_NabuStages.PHASE)
        self.setLayout(qt.QGridLayout())

        # unsharp coeff
        self._unsharpCoeffCB = qt.QCheckBox('unsharp coeff', self)
        self.layout().addWidget(self._unsharpCoeffCB, 0, 0, 1, 1)
        self._unsharpCoeffQLE = qt.QLineEdit('', self)
        self._unsharpCoeffQLE.setValidator(qt.QDoubleValidator())
        self.layout().addWidget(self._unsharpCoeffQLE, 0, 1, 1, 1)
        self.registerWidget(self._unsharpCoeffCB, 'optional')
        self._unsharpCoeffOpt = self.registerWidget(self._unsharpCoeffQLE, 'optional')

        # unsharp_sigma
        self._unsharpSigmaCB = qt.QCheckBox('unsharp sigma', self)
        self.layout().addWidget(self._unsharpSigmaCB, 1, 0, 1, 1)
        self._unsharpSigmaQLE = qt.QLineEdit('', self)
        self._unsharpSigmaQLE.setValidator(qt.QDoubleValidator())
        self.layout().addWidget(self._unsharpSigmaQLE, 1, 1, 1, 1)
        self.registerWidget(self._unsharpSigmaCB, 'optional')
        self._unsharpSigmaOpt = self.registerWidget(self._unsharpSigmaQLE, 'optional')

        # set up
        self._unsharpCoeffCB.setChecked(False)
        self._unsharpCoeffQLE.setText(str(0.0))
        self._unsharpSigmaCB.setChecked(False)
        self._unsharpSigmaQLE.setText(str(0.0))
        self._showUnsharpCoeffQLE(False)
        self._showUnsharpSigmaQLE(False)

        # signal / slot connection
        ## unsharp coeff
        self._unsharpCoeffCB.toggled.connect(self._showUnsharpCoeffQLE)
        self._unsharpCoeffCB.toggled.connect(self._unsharpCoeffChanged)
        self._unsharpCoeffQLE.editingFinished.connect(self._unsharpCoeffChanged)
        ## unsharp sigma
        self._unsharpSigmaCB.toggled.connect(self._showUnsharpSigmaQLE)
        self._unsharpSigmaCB.toggled.connect(self._unsharpSigmaChanged)
        self._unsharpSigmaQLE.editingFinished.connect(self._unsharpSigmaChanged)

    def _unsharpCoeffChanged(self, *args, **kwargs):
        self._signalConfChanged('unsharp_coeff')

    def _unsharpSigmaChanged(self, *args, **kwargs):
        self._signalConfChanged('unsharp_sigma')

    def isUnsharpCoeffActive(self):
        return self._unsharpCoeffCB.isChecked()

    def isUnsharpSigmaActive(self):
        return self._unsharpSigmaCB.isChecked()

    def _showUnsharpCoeffQLE(self, visible):
        self._unsharpCoeffOpt.setVisible(visible)

    def _showUnsharpSigmaQLE(self, visible):
        self._unsharpSigmaOpt.setVisible(visible)

    def _signalConfChanged(self, param):
        self.sigConfChanged.emit(param)

    def getUnsharpCoeff(self) -> typing.Union[float, int]:
        if self.isUnsharpCoeffActive():
            return float(self._unsharpCoeffQLE.text())
        else:
            return 0

    def setUnsharpCoeff(self, coeff):
        if coeff == 0.0:
            self._unsharpCoeffCB.setChecked(False)
        else:
            self._unsharpCoeffCB.setChecked(True)
            self._unsharpCoeffQLE.setText(str(coeff))

    def getUnsharpSigma(self) -> typing.Union[float, int]:
        if self.isUnsharpSigmaActive():
            return float(self._unsharpSigmaQLE.text())
        else:
            return 0

    def setUnsharpSigma(self, sigma):
        if sigma == 0.0:
            self._unsharpSigmaCB.setChecked(False)
        else:
            self._unsharpSigmaCB.setChecked(True)
            self._unsharpSigmaQLE.setText(str(sigma))

    def getConfiguration(self):
        return {
            'unsharp_coeff': self.getUnsharpCoeff(),
            'unsharp_sigma': self.getUnsharpSigma(),
        }

    def setConfiguration(self, conf):
        if 'unsharp_coeff' in conf:
            self.setUnsharpCoeff(float(conf['unsharp_coeff']))
        if 'unsharp_sigma' in conf:
            self.setUnsharpSigma(float(conf['unsharp_sigma']))


if __name__ == '__main__':
    app = qt.QApplication([])
    widget = _NabuPhaseConfig(None)
    widget.show()
    print(widget.getConfiguration())
    app.exec_()